from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisEmployeeContribution")


@attr.s(auto_attribs=True)
class PapdisEmployeeContribution:
    """
    Attributes:
        employer_contributions_amount (Union[Unset, float]): [readonly]
        employer_contributions_percent (Union[Unset, float]): [readonly]
        employee_contributions_amount (Union[Unset, float]): [readonly]
        employee_contributions_percent (Union[Unset, float]): [readonly]
        additional_voluntary_contributions_amount (Union[Unset, float]): [readonly]
        additional_voluntary_contributions_percent (Union[Unset, float]): [readonly]
        salary_sacrifice_indicator (Union[Unset, bool]): [readonly]
    """

    employer_contributions_amount: Union[Unset, float] = UNSET
    employer_contributions_percent: Union[Unset, float] = UNSET
    employee_contributions_amount: Union[Unset, float] = UNSET
    employee_contributions_percent: Union[Unset, float] = UNSET
    additional_voluntary_contributions_amount: Union[Unset, float] = UNSET
    additional_voluntary_contributions_percent: Union[Unset, float] = UNSET
    salary_sacrifice_indicator: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employer_contributions_amount = self.employer_contributions_amount
        employer_contributions_percent = self.employer_contributions_percent
        employee_contributions_amount = self.employee_contributions_amount
        employee_contributions_percent = self.employee_contributions_percent
        additional_voluntary_contributions_amount = (
            self.additional_voluntary_contributions_amount
        )
        additional_voluntary_contributions_percent = (
            self.additional_voluntary_contributions_percent
        )
        salary_sacrifice_indicator = self.salary_sacrifice_indicator

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employer_contributions_amount is not UNSET:
            field_dict["employerContributionsAmount"] = employer_contributions_amount
        if employer_contributions_percent is not UNSET:
            field_dict["employerContributionsPercent"] = employer_contributions_percent
        if employee_contributions_amount is not UNSET:
            field_dict["employeeContributionsAmount"] = employee_contributions_amount
        if employee_contributions_percent is not UNSET:
            field_dict["employeeContributionsPercent"] = employee_contributions_percent
        if additional_voluntary_contributions_amount is not UNSET:
            field_dict[
                "additionalVoluntaryContributionsAmount"
            ] = additional_voluntary_contributions_amount
        if additional_voluntary_contributions_percent is not UNSET:
            field_dict[
                "additionalVoluntaryContributionsPercent"
            ] = additional_voluntary_contributions_percent
        if salary_sacrifice_indicator is not UNSET:
            field_dict["salarySacrificeIndicator"] = salary_sacrifice_indicator

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employer_contributions_amount = d.pop("employerContributionsAmount", UNSET)

        employer_contributions_percent = d.pop("employerContributionsPercent", UNSET)

        employee_contributions_amount = d.pop("employeeContributionsAmount", UNSET)

        employee_contributions_percent = d.pop("employeeContributionsPercent", UNSET)

        additional_voluntary_contributions_amount = d.pop(
            "additionalVoluntaryContributionsAmount", UNSET
        )

        additional_voluntary_contributions_percent = d.pop(
            "additionalVoluntaryContributionsPercent", UNSET
        )

        salary_sacrifice_indicator = d.pop("salarySacrificeIndicator", UNSET)

        papdis_employee_contribution = cls(
            employer_contributions_amount=employer_contributions_amount,
            employer_contributions_percent=employer_contributions_percent,
            employee_contributions_amount=employee_contributions_amount,
            employee_contributions_percent=employee_contributions_percent,
            additional_voluntary_contributions_amount=additional_voluntary_contributions_amount,
            additional_voluntary_contributions_percent=additional_voluntary_contributions_percent,
            salary_sacrifice_indicator=salary_sacrifice_indicator,
        )

        return papdis_employee_contribution



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.auto_enrolment import AutoEnrolment
from ..models.bank_details import BankDetails
from ..models.employee_status import EmployeeStatus
from ..models.employment_details import EmploymentDetails
from ..models.leave_settings import LeaveSettings
from ..models.pay_options import PayOptions
from ..models.personal_details import PersonalDetails
from ..models.right_to_work import RightToWork
from ..types import UNSET, Unset

T = TypeVar("T", bound="Employee")


@attr.s(auto_attribs=True)
class Employee:
    """
    Attributes:
        has_pension (Union[Unset, bool]):
        id (Union[Unset, str]): [readonly] The unique id of the object
        personal_details (Union[Unset, PersonalDetails]):
        employment_details (Union[Unset, EmploymentDetails]):
        auto_enrolment (Union[Unset, AutoEnrolment]):
        leave_settings (Union[Unset, LeaveSettings]):
        right_to_work (Union[Unset, RightToWork]):
        bank_details (Union[Unset, BankDetails]):
        tags (Union[Unset, None, List[str]]):
        pay_options (Union[Unset, PayOptions]): This object forms the basis of the Employees payment.
        status (Union[Unset, EmployeeStatus]): [readonly] Indicates whether the Employee is currently employed or not
        ae_not_enroled_warning (Union[Unset, bool]): [readonly] If true then the employee should be enrolled in an Auto
            Enrolment Qualifying pension but isn't
        source_system_id (Union[Unset, None, str]): [readonly] Can only be given a value when the employee is created.
            It can then never be changed.
            Used by external systems so they can store an immutable reference
    """

    has_pension: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET
    personal_details: Union[Unset, PersonalDetails] = UNSET
    employment_details: Union[Unset, EmploymentDetails] = UNSET
    auto_enrolment: Union[Unset, AutoEnrolment] = UNSET
    leave_settings: Union[Unset, LeaveSettings] = UNSET
    right_to_work: Union[Unset, RightToWork] = UNSET
    bank_details: Union[Unset, BankDetails] = UNSET
    tags: Union[Unset, None, List[str]] = UNSET
    pay_options: Union[Unset, PayOptions] = UNSET
    status: Union[Unset, EmployeeStatus] = UNSET
    ae_not_enroled_warning: Union[Unset, bool] = UNSET
    source_system_id: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        has_pension = self.has_pension
        id = self.id
        personal_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.personal_details, Unset):
            personal_details = self.personal_details.to_dict()

        employment_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employment_details, Unset):
            employment_details = self.employment_details.to_dict()

        auto_enrolment: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.auto_enrolment, Unset):
            auto_enrolment = self.auto_enrolment.to_dict()

        leave_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.leave_settings, Unset):
            leave_settings = self.leave_settings.to_dict()

        right_to_work: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.right_to_work, Unset):
            right_to_work = self.right_to_work.to_dict()

        bank_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bank_details, Unset):
            bank_details = self.bank_details.to_dict()

        tags: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            if self.tags is None:
                tags = None
            else:
                tags = self.tags

        pay_options: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pay_options, Unset):
            pay_options = self.pay_options.to_dict()

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        ae_not_enroled_warning = self.ae_not_enroled_warning
        source_system_id = self.source_system_id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if has_pension is not UNSET:
            field_dict["hasPension"] = has_pension
        if id is not UNSET:
            field_dict["id"] = id
        if personal_details is not UNSET:
            field_dict["personalDetails"] = personal_details
        if employment_details is not UNSET:
            field_dict["employmentDetails"] = employment_details
        if auto_enrolment is not UNSET:
            field_dict["autoEnrolment"] = auto_enrolment
        if leave_settings is not UNSET:
            field_dict["leaveSettings"] = leave_settings
        if right_to_work is not UNSET:
            field_dict["rightToWork"] = right_to_work
        if bank_details is not UNSET:
            field_dict["bankDetails"] = bank_details
        if tags is not UNSET:
            field_dict["tags"] = tags
        if pay_options is not UNSET:
            field_dict["payOptions"] = pay_options
        if status is not UNSET:
            field_dict["status"] = status
        if ae_not_enroled_warning is not UNSET:
            field_dict["aeNotEnroledWarning"] = ae_not_enroled_warning
        if source_system_id is not UNSET:
            field_dict["sourceSystemId"] = source_system_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        has_pension = d.pop("hasPension", UNSET)

        id = d.pop("id", UNSET)

        _personal_details = d.pop("personalDetails", UNSET)
        personal_details: Union[Unset, PersonalDetails]
        if isinstance(_personal_details, Unset):
            personal_details = UNSET
        else:
            personal_details = PersonalDetails.from_dict(_personal_details)

        _employment_details = d.pop("employmentDetails", UNSET)
        employment_details: Union[Unset, EmploymentDetails]
        if isinstance(_employment_details, Unset):
            employment_details = UNSET
        else:
            employment_details = EmploymentDetails.from_dict(_employment_details)

        _auto_enrolment = d.pop("autoEnrolment", UNSET)
        auto_enrolment: Union[Unset, AutoEnrolment]
        if isinstance(_auto_enrolment, Unset):
            auto_enrolment = UNSET
        else:
            auto_enrolment = AutoEnrolment.from_dict(_auto_enrolment)

        _leave_settings = d.pop("leaveSettings", UNSET)
        leave_settings: Union[Unset, LeaveSettings]
        if isinstance(_leave_settings, Unset):
            leave_settings = UNSET
        else:
            leave_settings = LeaveSettings.from_dict(_leave_settings)

        _right_to_work = d.pop("rightToWork", UNSET)
        right_to_work: Union[Unset, RightToWork]
        if isinstance(_right_to_work, Unset):
            right_to_work = UNSET
        else:
            right_to_work = RightToWork.from_dict(_right_to_work)

        _bank_details = d.pop("bankDetails", UNSET)
        bank_details: Union[Unset, BankDetails]
        if isinstance(_bank_details, Unset):
            bank_details = UNSET
        else:
            bank_details = BankDetails.from_dict(_bank_details)

        tags = cast(List[str], d.pop("tags", UNSET))

        _pay_options = d.pop("payOptions", UNSET)
        pay_options: Union[Unset, PayOptions]
        if isinstance(_pay_options, Unset):
            pay_options = UNSET
        else:
            pay_options = PayOptions.from_dict(_pay_options)

        _status = d.pop("status", UNSET)
        status: Union[Unset, EmployeeStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = EmployeeStatus(_status)

        ae_not_enroled_warning = d.pop("aeNotEnroledWarning", UNSET)

        source_system_id = d.pop("sourceSystemId", UNSET)

        employee = cls(
            has_pension=has_pension,
            id=id,
            personal_details=personal_details,
            employment_details=employment_details,
            auto_enrolment=auto_enrolment,
            leave_settings=leave_settings,
            right_to_work=right_to_work,
            bank_details=bank_details,
            tags=tags,
            pay_options=pay_options,
            status=status,
            ae_not_enroled_warning=ae_not_enroled_warning,
            source_system_id=source_system_id,
        )

        return employee



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FixedCode")


@attr.s(auto_attribs=True)
class FixedCode:
    """Part of the TaxYearConfig that our engine uses to enable calculation of Tax and NI.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            code (Union[Unset, None, str]): [readonly]
            rate (Union[Unset, float]): [readonly]
    """

    code: Union[Unset, None, str] = UNSET
    rate: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        rate = self.rate

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if code is not UNSET:
            field_dict["code"] = code
        if rate is not UNSET:
            field_dict["rate"] = rate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code", UNSET)

        rate = d.pop("rate", UNSET)

        fixed_code = cls(
            code=code,
            rate=rate,
        )

        return fixed_code



from enum import Enum


class Report(str, Enum):
    GROSSTONET = "GrossToNet"
    GROSSTONETCIS = "GrossToNetCis"
    PAYRUNPAYMENTS = "PayrunPayments"
    FURLOUGH = "Furlough"
    PENSIONCONTRIBS = "PensionContribs"
    JOURNAL = "Journal"
    HOURLYPAY = "HourlyPay"
    UMBRELLARECONCILIATION = "UmbrellaReconciliation"
    UMBRELLASTATEMENT = "UmbrellaStatement"
    STATUTORYPAY = "StatutoryPay"
    COSTANALYSIS = "CostAnalysis"
    COSTOFEMPLOYMENT = "CostOfEmployment"
    FULLSUMMARYOFPAY = "FullSummaryOfPay"
    P11 = "P11"
    P11D = "P11d"
    P30 = "P30"
    P32 = "P32"
    P45 = "P45"
    P60 = "P60"
    EMPLOYEEEXPORT = "EmployeeExport"
    RIGHTTOWORK = "RightToWork"
    AEOSTATEMENT = "AeoStatement"
    HOLIDAYALLOWANCES = "HolidayAllowances"
    HOLIDAYPAYACCRUALS = "HolidayPayAccruals"
    AEASSESSMENTS = "AeAssessments"
    CISSTATEMENT = "CisStatement"
    SUBCONTRACTORSUMMARY = "SubcontractorSummary"
    PAYROLLANALYSIS = "PayrollAnalysis"
    VARIANCEREPORT = "VarianceReport"
    P11DETAILED = "P11Detailed"
    NILETTERVALIDATION = "NiLetterValidation"
    YTD = "Ytd"
    TAXCODECHANGES = "TaxCodeChanges"
    EMPLOYEEBENEFITS = "EmployeeBenefits"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="HtmlEmailSettings")


@attr.s(auto_attribs=True)
class HtmlEmailSettings:
    """
    Attributes:
        body_css_style (Union[Unset, None, str]):
        font_css_style (Union[Unset, None, str]):
        button_css_style (Union[Unset, None, str]):
        header_html (Union[Unset, None, str]):
        footer_html (Union[Unset, None, str]):
    """

    body_css_style: Union[Unset, None, str] = UNSET
    font_css_style: Union[Unset, None, str] = UNSET
    button_css_style: Union[Unset, None, str] = UNSET
    header_html: Union[Unset, None, str] = UNSET
    footer_html: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        body_css_style = self.body_css_style
        font_css_style = self.font_css_style
        button_css_style = self.button_css_style
        header_html = self.header_html
        footer_html = self.footer_html

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if body_css_style is not UNSET:
            field_dict["bodyCssStyle"] = body_css_style
        if font_css_style is not UNSET:
            field_dict["fontCssStyle"] = font_css_style
        if button_css_style is not UNSET:
            field_dict["buttonCssStyle"] = button_css_style
        if header_html is not UNSET:
            field_dict["headerHtml"] = header_html
        if footer_html is not UNSET:
            field_dict["footerHtml"] = footer_html

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        body_css_style = d.pop("bodyCssStyle", UNSET)

        font_css_style = d.pop("fontCssStyle", UNSET)

        button_css_style = d.pop("buttonCssStyle", UNSET)

        header_html = d.pop("headerHtml", UNSET)

        footer_html = d.pop("footerHtml", UNSET)

        html_email_settings = cls(
            body_css_style=body_css_style,
            font_css_style=font_css_style,
            button_css_style=button_css_style,
            header_html=header_html,
            footer_html=footer_html,
        )

        return html_email_settings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployeeTrivialCommutationPayment")


@attr.s(auto_attribs=True)
class FpsEmployeeTrivialCommutationPayment:
    """
    Attributes:
        type (Union[Unset, None, str]):
        value (Union[Unset, None, str]):
    """

    type: Union[Unset, None, str] = UNSET
    value: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        value = self.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", UNSET)

        value = d.pop("value", UNSET)

        fps_employee_trivial_commutation_payment = cls(
            type=type,
            value=value,
        )

        return fps_employee_trivial_commutation_payment



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.address import Address
from ..models.direct_debit_mandate import DirectDebitMandate
from ..models.monthly_minimum import MonthlyMinimum
from ..models.role import Role
from ..models.tenant import Tenant
from ..models.user_authorization import UserAuthorization
from ..models.user_category import UserCategory
from ..models.user_display_preferences import UserDisplayPreferences
from ..models.user_industry import UserIndustry
from ..models.user_job_type import UserJobType
from ..models.utm_info import UtmInfo
from ..types import UNSET, Unset

T = TypeVar("T", bound="User")


@attr.s(auto_attribs=True)
class User:
    """Represents a User Account.
    As well as basic details about the user it also includes details of Employers that the user account can access.

        Attributes:
            category (Union[Unset, UserCategory]):
            email_address (Union[Unset, None, str]):
            pending_email_address (Union[Unset, None, str]): If the user has requested to change their email address then
                the
                address it'll be changed to after verification will be shown here.
            first_name (Union[Unset, None, str]):
            last_name (Union[Unset, None, str]):
            salutation (Union[Unset, None, str]):
            user_identifier (Union[Unset, None, str]):
            photo (Union[Unset, None, str]):
            role (Union[Unset, Role]):
            job_type (Union[Unset, UserJobType]):
            job_title (Union[Unset, None, str]):
            telephone_number (Union[Unset, None, str]):
            business_name (Union[Unset, None, str]):
            industry (Union[Unset, UserIndustry]):
            address (Union[Unset, Address]):
            stated_employee_count (Union[Unset, None, int]):
            email_verified (Union[Unset, bool]):
            gdpr_optin (Union[Unset, bool]):
            invite_code (Union[Unset, None, str]):
            registration_ip (Union[Unset, None, str]):
            registration_date (Union[Unset, datetime.date]):
            last_login (Union[Unset, None, datetime.date]):
            is_activated (Union[Unset, bool]):
            authorization (Union[Unset, UserAuthorization]): This model provides information about what the User is able to
                access.
                This would usually just be a list of Employers. But if the user is an administrator for a White Label instance
                then this will be shown in the list of Tenants.
            is_super_admin (Union[Unset, bool]):
            can_use_bureau_features (Union[Unset, bool]):
            omnipresent_role (Union[Unset, Role]):
            tenant (Union[Unset, Tenant]): The Tenant model represents the brand that provides the account.
                This is used by our White Label partners to manage and brand their user accounts.
                Unless you are an admin for a White Label account you'll have no interest in this model.
            request_dd_setup (Union[Unset, bool]): [readonly] If true then the user is required to set up a direct debit
                mandate
            disabled (Union[Unset, bool]): [readonly] If true then any employers the owner managed will not be able to run
                new payruns.
                DisabledReason will give a reason why the account is disabled
            can_create_employers (Union[Unset, bool]): [readonly] If false then the user cannot create new employers.
                This can be turned on or off by the tenant admin.
            disabled_reason (Union[Unset, None, str]):
            direct_debit_mandate (Union[Unset, DirectDebitMandate]):
            display_prefs (Union[Unset, UserDisplayPreferences]):
            show_bills (Union[Unset, None, bool]): [readonly] Whether or not the user can see bills. This will be false if
                the Tenant manages billing and the user is not an admin for the Tenant
            accounting_customer_id (Union[Unset, None, str]): [readonly] Used internally to manage billing
            utm_info (Union[Unset, UtmInfo]):
            first_billable_activity_date (Union[Unset, None, datetime.date]):
            bureau_notification_email_address (Union[Unset, None, str]): If an email address is provided here then Bureau-
                related notifications will go to this address instead of the EmailAddress
            monthly_minimum (Union[Unset, MonthlyMinimum]):
            id (Union[Unset, str]): [readonly] The unique id of the object
    """

    category: Union[Unset, UserCategory] = UNSET
    email_address: Union[Unset, None, str] = UNSET
    pending_email_address: Union[Unset, None, str] = UNSET
    first_name: Union[Unset, None, str] = UNSET
    last_name: Union[Unset, None, str] = UNSET
    salutation: Union[Unset, None, str] = UNSET
    user_identifier: Union[Unset, None, str] = UNSET
    photo: Union[Unset, None, str] = UNSET
    role: Union[Unset, Role] = UNSET
    job_type: Union[Unset, UserJobType] = UNSET
    job_title: Union[Unset, None, str] = UNSET
    telephone_number: Union[Unset, None, str] = UNSET
    business_name: Union[Unset, None, str] = UNSET
    industry: Union[Unset, UserIndustry] = UNSET
    address: Union[Unset, Address] = UNSET
    stated_employee_count: Union[Unset, None, int] = UNSET
    email_verified: Union[Unset, bool] = UNSET
    gdpr_optin: Union[Unset, bool] = UNSET
    invite_code: Union[Unset, None, str] = UNSET
    registration_ip: Union[Unset, None, str] = UNSET
    registration_date: Union[Unset, datetime.date] = UNSET
    last_login: Union[Unset, None, datetime.date] = UNSET
    is_activated: Union[Unset, bool] = UNSET
    authorization: Union[Unset, UserAuthorization] = UNSET
    is_super_admin: Union[Unset, bool] = UNSET
    can_use_bureau_features: Union[Unset, bool] = UNSET
    omnipresent_role: Union[Unset, Role] = UNSET
    tenant: Union[Unset, Tenant] = UNSET
    request_dd_setup: Union[Unset, bool] = UNSET
    disabled: Union[Unset, bool] = UNSET
    can_create_employers: Union[Unset, bool] = UNSET
    disabled_reason: Union[Unset, None, str] = UNSET
    direct_debit_mandate: Union[Unset, DirectDebitMandate] = UNSET
    display_prefs: Union[Unset, UserDisplayPreferences] = UNSET
    show_bills: Union[Unset, None, bool] = UNSET
    accounting_customer_id: Union[Unset, None, str] = UNSET
    utm_info: Union[Unset, UtmInfo] = UNSET
    first_billable_activity_date: Union[Unset, None, datetime.date] = UNSET
    bureau_notification_email_address: Union[Unset, None, str] = UNSET
    monthly_minimum: Union[Unset, MonthlyMinimum] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        category: Union[Unset, str] = UNSET
        if not isinstance(self.category, Unset):
            category = self.category.value

        email_address = self.email_address
        pending_email_address = self.pending_email_address
        first_name = self.first_name
        last_name = self.last_name
        salutation = self.salutation
        user_identifier = self.user_identifier
        photo = self.photo
        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        job_type: Union[Unset, str] = UNSET
        if not isinstance(self.job_type, Unset):
            job_type = self.job_type.value

        job_title = self.job_title
        telephone_number = self.telephone_number
        business_name = self.business_name
        industry: Union[Unset, str] = UNSET
        if not isinstance(self.industry, Unset):
            industry = self.industry.value

        address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        stated_employee_count = self.stated_employee_count
        email_verified = self.email_verified
        gdpr_optin = self.gdpr_optin
        invite_code = self.invite_code
        registration_ip = self.registration_ip
        registration_date: Union[Unset, str] = UNSET
        if not isinstance(self.registration_date, Unset):
            registration_date = self.registration_date.isoformat()

        last_login: Union[Unset, None, str] = UNSET
        if not isinstance(self.last_login, Unset):
            last_login = self.last_login.isoformat() if self.last_login else None

        is_activated = self.is_activated
        authorization: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.authorization, Unset):
            authorization = self.authorization.to_dict()

        is_super_admin = self.is_super_admin
        can_use_bureau_features = self.can_use_bureau_features
        omnipresent_role: Union[Unset, str] = UNSET
        if not isinstance(self.omnipresent_role, Unset):
            omnipresent_role = self.omnipresent_role.value

        tenant: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tenant, Unset):
            tenant = self.tenant.to_dict()

        request_dd_setup = self.request_dd_setup
        disabled = self.disabled
        can_create_employers = self.can_create_employers
        disabled_reason = self.disabled_reason
        direct_debit_mandate: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.direct_debit_mandate, Unset):
            direct_debit_mandate = self.direct_debit_mandate.to_dict()

        display_prefs: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.display_prefs, Unset):
            display_prefs = self.display_prefs.to_dict()

        show_bills = self.show_bills
        accounting_customer_id = self.accounting_customer_id
        utm_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.utm_info, Unset):
            utm_info = self.utm_info.to_dict()

        first_billable_activity_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.first_billable_activity_date, Unset):
            first_billable_activity_date = (
                self.first_billable_activity_date.isoformat()
                if self.first_billable_activity_date
                else None
            )

        bureau_notification_email_address = self.bureau_notification_email_address
        monthly_minimum: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_minimum, Unset):
            monthly_minimum = self.monthly_minimum.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if category is not UNSET:
            field_dict["category"] = category
        if email_address is not UNSET:
            field_dict["emailAddress"] = email_address
        if pending_email_address is not UNSET:
            field_dict["pendingEmailAddress"] = pending_email_address
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if salutation is not UNSET:
            field_dict["salutation"] = salutation
        if user_identifier is not UNSET:
            field_dict["userIdentifier"] = user_identifier
        if photo is not UNSET:
            field_dict["photo"] = photo
        if role is not UNSET:
            field_dict["role"] = role
        if job_type is not UNSET:
            field_dict["jobType"] = job_type
        if job_title is not UNSET:
            field_dict["jobTitle"] = job_title
        if telephone_number is not UNSET:
            field_dict["telephoneNumber"] = telephone_number
        if business_name is not UNSET:
            field_dict["businessName"] = business_name
        if industry is not UNSET:
            field_dict["industry"] = industry
        if address is not UNSET:
            field_dict["address"] = address
        if stated_employee_count is not UNSET:
            field_dict["statedEmployeeCount"] = stated_employee_count
        if email_verified is not UNSET:
            field_dict["emailVerified"] = email_verified
        if gdpr_optin is not UNSET:
            field_dict["gdprOptin"] = gdpr_optin
        if invite_code is not UNSET:
            field_dict["inviteCode"] = invite_code
        if registration_ip is not UNSET:
            field_dict["registrationIp"] = registration_ip
        if registration_date is not UNSET:
            field_dict["registrationDate"] = registration_date
        if last_login is not UNSET:
            field_dict["lastLogin"] = last_login
        if is_activated is not UNSET:
            field_dict["isActivated"] = is_activated
        if authorization is not UNSET:
            field_dict["authorization"] = authorization
        if is_super_admin is not UNSET:
            field_dict["isSuperAdmin"] = is_super_admin
        if can_use_bureau_features is not UNSET:
            field_dict["canUseBureauFeatures"] = can_use_bureau_features
        if omnipresent_role is not UNSET:
            field_dict["omnipresentRole"] = omnipresent_role
        if tenant is not UNSET:
            field_dict["tenant"] = tenant
        if request_dd_setup is not UNSET:
            field_dict["requestDdSetup"] = request_dd_setup
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if can_create_employers is not UNSET:
            field_dict["canCreateEmployers"] = can_create_employers
        if disabled_reason is not UNSET:
            field_dict["disabledReason"] = disabled_reason
        if direct_debit_mandate is not UNSET:
            field_dict["directDebitMandate"] = direct_debit_mandate
        if display_prefs is not UNSET:
            field_dict["displayPrefs"] = display_prefs
        if show_bills is not UNSET:
            field_dict["showBills"] = show_bills
        if accounting_customer_id is not UNSET:
            field_dict["accountingCustomerId"] = accounting_customer_id
        if utm_info is not UNSET:
            field_dict["utmInfo"] = utm_info
        if first_billable_activity_date is not UNSET:
            field_dict["firstBillableActivityDate"] = first_billable_activity_date
        if bureau_notification_email_address is not UNSET:
            field_dict[
                "bureauNotificationEmailAddress"
            ] = bureau_notification_email_address
        if monthly_minimum is not UNSET:
            field_dict["monthlyMinimum"] = monthly_minimum
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _category = d.pop("category", UNSET)
        category: Union[Unset, UserCategory]
        if isinstance(_category, Unset):
            category = UNSET
        else:
            category = UserCategory(_category)

        email_address = d.pop("emailAddress", UNSET)

        pending_email_address = d.pop("pendingEmailAddress", UNSET)

        first_name = d.pop("firstName", UNSET)

        last_name = d.pop("lastName", UNSET)

        salutation = d.pop("salutation", UNSET)

        user_identifier = d.pop("userIdentifier", UNSET)

        photo = d.pop("photo", UNSET)

        _role = d.pop("role", UNSET)
        role: Union[Unset, Role]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = Role(_role)

        _job_type = d.pop("jobType", UNSET)
        job_type: Union[Unset, UserJobType]
        if isinstance(_job_type, Unset):
            job_type = UNSET
        else:
            job_type = UserJobType(_job_type)

        job_title = d.pop("jobTitle", UNSET)

        telephone_number = d.pop("telephoneNumber", UNSET)

        business_name = d.pop("businessName", UNSET)

        _industry = d.pop("industry", UNSET)
        industry: Union[Unset, UserIndustry]
        if isinstance(_industry, Unset):
            industry = UNSET
        else:
            industry = UserIndustry(_industry)

        _address = d.pop("address", UNSET)
        address: Union[Unset, Address]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = Address.from_dict(_address)

        stated_employee_count = d.pop("statedEmployeeCount", UNSET)

        email_verified = d.pop("emailVerified", UNSET)

        gdpr_optin = d.pop("gdprOptin", UNSET)

        invite_code = d.pop("inviteCode", UNSET)

        registration_ip = d.pop("registrationIp", UNSET)

        _registration_date = d.pop("registrationDate", UNSET)
        registration_date: Union[Unset, datetime.date]
        if isinstance(_registration_date, Unset):
            registration_date = UNSET
        else:
            registration_date = isoparse(_registration_date).date()

        _last_login = d.pop("lastLogin", UNSET)
        last_login: Union[Unset, None, datetime.date]
        if _last_login is None:
            last_login = None
        elif isinstance(_last_login, Unset):
            last_login = UNSET
        else:
            last_login = isoparse(_last_login).date()

        is_activated = d.pop("isActivated", UNSET)

        _authorization = d.pop("authorization", UNSET)
        authorization: Union[Unset, UserAuthorization]
        if isinstance(_authorization, Unset):
            authorization = UNSET
        else:
            authorization = UserAuthorization.from_dict(_authorization)

        is_super_admin = d.pop("isSuperAdmin", UNSET)

        can_use_bureau_features = d.pop("canUseBureauFeatures", UNSET)

        _omnipresent_role = d.pop("omnipresentRole", UNSET)
        omnipresent_role: Union[Unset, Role]
        if isinstance(_omnipresent_role, Unset):
            omnipresent_role = UNSET
        else:
            omnipresent_role = Role(_omnipresent_role)

        _tenant = d.pop("tenant", UNSET)
        tenant: Union[Unset, Tenant]
        if isinstance(_tenant, Unset):
            tenant = UNSET
        else:
            tenant = Tenant.from_dict(_tenant)

        request_dd_setup = d.pop("requestDdSetup", UNSET)

        disabled = d.pop("disabled", UNSET)

        can_create_employers = d.pop("canCreateEmployers", UNSET)

        disabled_reason = d.pop("disabledReason", UNSET)

        _direct_debit_mandate = d.pop("directDebitMandate", UNSET)
        direct_debit_mandate: Union[Unset, DirectDebitMandate]
        if isinstance(_direct_debit_mandate, Unset):
            direct_debit_mandate = UNSET
        else:
            direct_debit_mandate = DirectDebitMandate.from_dict(_direct_debit_mandate)

        _display_prefs = d.pop("displayPrefs", UNSET)
        display_prefs: Union[Unset, UserDisplayPreferences]
        if isinstance(_display_prefs, Unset):
            display_prefs = UNSET
        else:
            display_prefs = UserDisplayPreferences.from_dict(_display_prefs)

        show_bills = d.pop("showBills", UNSET)

        accounting_customer_id = d.pop("accountingCustomerId", UNSET)

        _utm_info = d.pop("utmInfo", UNSET)
        utm_info: Union[Unset, UtmInfo]
        if isinstance(_utm_info, Unset):
            utm_info = UNSET
        else:
            utm_info = UtmInfo.from_dict(_utm_info)

        _first_billable_activity_date = d.pop("firstBillableActivityDate", UNSET)
        first_billable_activity_date: Union[Unset, None, datetime.date]
        if _first_billable_activity_date is None:
            first_billable_activity_date = None
        elif isinstance(_first_billable_activity_date, Unset):
            first_billable_activity_date = UNSET
        else:
            first_billable_activity_date = isoparse(
                _first_billable_activity_date
            ).date()

        bureau_notification_email_address = d.pop(
            "bureauNotificationEmailAddress", UNSET
        )

        _monthly_minimum = d.pop("monthlyMinimum", UNSET)
        monthly_minimum: Union[Unset, MonthlyMinimum]
        if isinstance(_monthly_minimum, Unset):
            monthly_minimum = UNSET
        else:
            monthly_minimum = MonthlyMinimum.from_dict(_monthly_minimum)

        id = d.pop("id", UNSET)

        user = cls(
            category=category,
            email_address=email_address,
            pending_email_address=pending_email_address,
            first_name=first_name,
            last_name=last_name,
            salutation=salutation,
            user_identifier=user_identifier,
            photo=photo,
            role=role,
            job_type=job_type,
            job_title=job_title,
            telephone_number=telephone_number,
            business_name=business_name,
            industry=industry,
            address=address,
            stated_employee_count=stated_employee_count,
            email_verified=email_verified,
            gdpr_optin=gdpr_optin,
            invite_code=invite_code,
            registration_ip=registration_ip,
            registration_date=registration_date,
            last_login=last_login,
            is_activated=is_activated,
            authorization=authorization,
            is_super_admin=is_super_admin,
            can_use_bureau_features=can_use_bureau_features,
            omnipresent_role=omnipresent_role,
            tenant=tenant,
            request_dd_setup=request_dd_setup,
            disabled=disabled,
            can_create_employers=can_create_employers,
            disabled_reason=disabled_reason,
            direct_debit_mandate=direct_debit_mandate,
            display_prefs=display_prefs,
            show_bills=show_bills,
            accounting_customer_id=accounting_customer_id,
            utm_info=utm_info,
            first_billable_activity_date=first_billable_activity_date,
            bureau_notification_email_address=bureau_notification_email_address,
            monthly_minimum=monthly_minimum,
            id=id,
        )

        return user



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="P11PayeTotalsLine")


@attr.s(auto_attribs=True)
class P11PayeTotalsLine:
    """Summary line for the Paye Income Tax table in the P11 Detailed report

    Attributes:
        student_loan (Union[Unset, float]): [readonly]
        postgrad_loan (Union[Unset, float]): [readonly]
        pay (Union[Unset, float]): [readonly]
        total_pay_to_date (Union[Unset, float]): [readonly]
        tax_due_to_date (Union[Unset, float]): [readonly]
        tax_due (Union[Unset, float]): [readonly]
    """

    student_loan: Union[Unset, float] = UNSET
    postgrad_loan: Union[Unset, float] = UNSET
    pay: Union[Unset, float] = UNSET
    total_pay_to_date: Union[Unset, float] = UNSET
    tax_due_to_date: Union[Unset, float] = UNSET
    tax_due: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        student_loan = self.student_loan
        postgrad_loan = self.postgrad_loan
        pay = self.pay
        total_pay_to_date = self.total_pay_to_date
        tax_due_to_date = self.tax_due_to_date
        tax_due = self.tax_due

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if student_loan is not UNSET:
            field_dict["studentLoan"] = student_loan
        if postgrad_loan is not UNSET:
            field_dict["postgradLoan"] = postgrad_loan
        if pay is not UNSET:
            field_dict["pay"] = pay
        if total_pay_to_date is not UNSET:
            field_dict["totalPayToDate"] = total_pay_to_date
        if tax_due_to_date is not UNSET:
            field_dict["taxDueToDate"] = tax_due_to_date
        if tax_due is not UNSET:
            field_dict["taxDue"] = tax_due

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        student_loan = d.pop("studentLoan", UNSET)

        postgrad_loan = d.pop("postgradLoan", UNSET)

        pay = d.pop("pay", UNSET)

        total_pay_to_date = d.pop("totalPayToDate", UNSET)

        tax_due_to_date = d.pop("taxDueToDate", UNSET)

        tax_due = d.pop("taxDue", UNSET)

        p11_paye_totals_line = cls(
            student_loan=student_loan,
            postgrad_loan=postgrad_loan,
            pay=pay,
            total_pay_to_date=total_pay_to_date,
            tax_due_to_date=tax_due_to_date,
            tax_due=tax_due,
        )

        return p11_paye_totals_line



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.p11d_desc_other import P11DDescOther
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DOther")


@attr.s(auto_attribs=True)
class P11DOther:
    """
    Attributes:
        class_1a (Union[Unset, None, List[P11DDescOther]]):
        non_class_1a (Union[Unset, None, List[P11DDescOther]]):
        tax_paid (Union[Unset, None, str]):
        type_letter (Union[Unset, None, str]):
    """

    class_1a: Union[Unset, None, List[P11DDescOther]] = UNSET
    non_class_1a: Union[Unset, None, List[P11DDescOther]] = UNSET
    tax_paid: Union[Unset, None, str] = UNSET
    type_letter: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        class_1a: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.class_1a, Unset):
            if self.class_1a is None:
                class_1a = None
            else:
                class_1a = []
                for class_1a_item_data in self.class_1a:
                    class_1a_item = class_1a_item_data.to_dict()

                    class_1a.append(class_1a_item)

        non_class_1a: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.non_class_1a, Unset):
            if self.non_class_1a is None:
                non_class_1a = None
            else:
                non_class_1a = []
                for non_class_1a_item_data in self.non_class_1a:
                    non_class_1a_item = non_class_1a_item_data.to_dict()

                    non_class_1a.append(non_class_1a_item)

        tax_paid = self.tax_paid
        type_letter = self.type_letter

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if class_1a is not UNSET:
            field_dict["class1A"] = class_1a
        if non_class_1a is not UNSET:
            field_dict["nonClass1A"] = non_class_1a
        if tax_paid is not UNSET:
            field_dict["taxPaid"] = tax_paid
        if type_letter is not UNSET:
            field_dict["typeLetter"] = type_letter

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        class_1a = []
        _class_1a = d.pop("class1A", UNSET)
        for class_1a_item_data in _class_1a or []:
            class_1a_item = P11DDescOther.from_dict(class_1a_item_data)

            class_1a.append(class_1a_item)

        non_class_1a = []
        _non_class_1a = d.pop("nonClass1A", UNSET)
        for non_class_1a_item_data in _non_class_1a or []:
            non_class_1a_item = P11DDescOther.from_dict(non_class_1a_item_data)

            non_class_1a.append(non_class_1a_item)

        tax_paid = d.pop("taxPaid", UNSET)

        type_letter = d.pop("typeLetter", UNSET)

        p11d_other = cls(
            class_1a=class_1a,
            non_class_1a=non_class_1a,
            tax_paid=tax_paid,
            type_letter=type_letter,
        )

        return p11d_other



from enum import Enum


class PayeeType(str, Enum):
    EMPLOYEE = "Employee"
    HMRC = "Hmrc"
    PENSIONPROVIDER = "PensionProvider"
    AEO = "Aeo"
    DEDUCTION = "Deduction"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from enum import Enum


class WebhookEvent(str, Enum):
    EMPLOYEE_CREATED = "Employee_Created"
    EMPLOYEE_UPDATED = "Employee_Updated"
    EMPLOYEE_DELETED = "Employee_Deleted"
    PAYRUN_FINALISED = "Payrun_Finalised"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="VeteranDetails")


@attr.s(auto_attribs=True)
class VeteranDetails:
    """Employment details for veterans

    Attributes:
        is_veteran (Union[Unset, bool]): Set to true if the employee is a veteran
        first_civilian_employment_date (Union[Unset, None, datetime.date]): Date of Veteran's first civilian employment
    """

    is_veteran: Union[Unset, bool] = UNSET
    first_civilian_employment_date: Union[Unset, None, datetime.date] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_veteran = self.is_veteran
        first_civilian_employment_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.first_civilian_employment_date, Unset):
            first_civilian_employment_date = (
                self.first_civilian_employment_date.isoformat()
                if self.first_civilian_employment_date
                else None
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_veteran is not UNSET:
            field_dict["isVeteran"] = is_veteran
        if first_civilian_employment_date is not UNSET:
            field_dict["firstCivilianEmploymentDate"] = first_civilian_employment_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_veteran = d.pop("isVeteran", UNSET)

        _first_civilian_employment_date = d.pop("firstCivilianEmploymentDate", UNSET)
        first_civilian_employment_date: Union[Unset, None, datetime.date]
        if _first_civilian_employment_date is None:
            first_civilian_employment_date = None
        elif isinstance(_first_civilian_employment_date, Unset):
            first_civilian_employment_date = UNSET
        else:
            first_civilian_employment_date = isoparse(
                _first_civilian_employment_date
            ).date()

        veteran_details = cls(
            is_veteran=is_veteran,
            first_civilian_employment_date=first_civilian_employment_date,
        )

        return veteran_details



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.fps_fields import FpsFields
from ..models.mileage_vehicle_type import MileageVehicleType
from ..models.pay_basis import PayBasis
from ..models.pay_line import PayLine
from ..models.pay_method import PayMethod
from ..models.pay_periods import PayPeriods
from ..models.tax_and_ni import TaxAndNi
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayOptions")


@attr.s(auto_attribs=True)
class PayOptions:
    """This object forms the basis of the Employees payment.

    Attributes:
        period (Union[Unset, PayPeriods]):
        ordinal (Union[Unset, int]): Indicates whether to use first, second, third (etc) PaySchedule for this PayPeriod.
        pay_amount (Union[Unset, float]): The amount the Employee is regularly paid each period
        basis (Union[Unset, PayBasis]):
        national_minimum_wage (Union[Unset, bool]):
        pay_amount_multiplier (Union[Unset, float]): This property is irrelevant if the basis is Monthly.
            But if the basis is Daily or Hourly then this property sets how many days/hours the employee should be paid for
            in the period.
        base_hourly_rate (Union[Unset, float]): This property is used to calculate values for PayCodes that are set as
            multiples of
            the employees base hourly rate. Eg Overtime.
            If this is set as zero then we'll attempt to calculate a value based on the other fields
        base_daily_rate (Union[Unset, float]): This property is used to calculate values for PayCodes that are set as
            multiples of
            the employees base daily rate. Eg sick.
            If this is set as zero then we'll attempt to calculate a value based on the other fields
        auto_adjust_for_leave (Union[Unset, bool]): Automatically reduce the PayAmount when the Employee has Leave that
            is either Not Paid or has Statutory Pay.
            Can only be set to True if the Basis is Monthly (ie, employee is not paid an hourly or daily rate).
            If set to false then you must manually reduce their payment to reflect any Leave
        method (Union[Unset, PayMethod]):
        pay_code (Union[Unset, None, str]): If you want the Employees pay to be allocated to a code other than BASIC,
            specify it here
        withhold_tax_refund_if_pay_is_zero (Union[Unset, bool]): If you don't want negative PAYE values when the Gross
            pay is zero, set this to true
        mileage_vehicle_type (Union[Unset, MileageVehicleType]):
        maps_miles (Union[Unset, None, int]): The number of miles to pay for as Mileage Allowance Payments
        tax_and_ni (Union[Unset, TaxAndNi]):
        fps_fields (Union[Unset, FpsFields]):
        regular_pay_lines (Union[Unset, None, List[PayLine]]): These are used to make additions/deductions to the pay
            for this Employee.
            You do not need to include Pension, Tax, NI,  Loan Repayments, etc as these will all be automatically created.
        tags (Union[Unset, None, List[str]]):
    """

    period: Union[Unset, PayPeriods] = UNSET
    ordinal: Union[Unset, int] = UNSET
    pay_amount: Union[Unset, float] = UNSET
    basis: Union[Unset, PayBasis] = UNSET
    national_minimum_wage: Union[Unset, bool] = UNSET
    pay_amount_multiplier: Union[Unset, float] = UNSET
    base_hourly_rate: Union[Unset, float] = UNSET
    base_daily_rate: Union[Unset, float] = UNSET
    auto_adjust_for_leave: Union[Unset, bool] = UNSET
    method: Union[Unset, PayMethod] = UNSET
    pay_code: Union[Unset, None, str] = UNSET
    withhold_tax_refund_if_pay_is_zero: Union[Unset, bool] = UNSET
    mileage_vehicle_type: Union[Unset, MileageVehicleType] = UNSET
    maps_miles: Union[Unset, None, int] = UNSET
    tax_and_ni: Union[Unset, TaxAndNi] = UNSET
    fps_fields: Union[Unset, FpsFields] = UNSET
    regular_pay_lines: Union[Unset, None, List[PayLine]] = UNSET
    tags: Union[Unset, None, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        period: Union[Unset, str] = UNSET
        if not isinstance(self.period, Unset):
            period = self.period.value

        ordinal = self.ordinal
        pay_amount = self.pay_amount
        basis: Union[Unset, str] = UNSET
        if not isinstance(self.basis, Unset):
            basis = self.basis.value

        national_minimum_wage = self.national_minimum_wage
        pay_amount_multiplier = self.pay_amount_multiplier
        base_hourly_rate = self.base_hourly_rate
        base_daily_rate = self.base_daily_rate
        auto_adjust_for_leave = self.auto_adjust_for_leave
        method: Union[Unset, str] = UNSET
        if not isinstance(self.method, Unset):
            method = self.method.value

        pay_code = self.pay_code
        withhold_tax_refund_if_pay_is_zero = self.withhold_tax_refund_if_pay_is_zero
        mileage_vehicle_type: Union[Unset, str] = UNSET
        if not isinstance(self.mileage_vehicle_type, Unset):
            mileage_vehicle_type = self.mileage_vehicle_type.value

        maps_miles = self.maps_miles
        tax_and_ni: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tax_and_ni, Unset):
            tax_and_ni = self.tax_and_ni.to_dict()

        fps_fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.fps_fields, Unset):
            fps_fields = self.fps_fields.to_dict()

        regular_pay_lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.regular_pay_lines, Unset):
            if self.regular_pay_lines is None:
                regular_pay_lines = None
            else:
                regular_pay_lines = []
                for regular_pay_lines_item_data in self.regular_pay_lines:
                    regular_pay_lines_item = regular_pay_lines_item_data.to_dict()

                    regular_pay_lines.append(regular_pay_lines_item)

        tags: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            if self.tags is None:
                tags = None
            else:
                tags = self.tags

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if period is not UNSET:
            field_dict["period"] = period
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if pay_amount is not UNSET:
            field_dict["payAmount"] = pay_amount
        if basis is not UNSET:
            field_dict["basis"] = basis
        if national_minimum_wage is not UNSET:
            field_dict["nationalMinimumWage"] = national_minimum_wage
        if pay_amount_multiplier is not UNSET:
            field_dict["payAmountMultiplier"] = pay_amount_multiplier
        if base_hourly_rate is not UNSET:
            field_dict["baseHourlyRate"] = base_hourly_rate
        if base_daily_rate is not UNSET:
            field_dict["baseDailyRate"] = base_daily_rate
        if auto_adjust_for_leave is not UNSET:
            field_dict["autoAdjustForLeave"] = auto_adjust_for_leave
        if method is not UNSET:
            field_dict["method"] = method
        if pay_code is not UNSET:
            field_dict["payCode"] = pay_code
        if withhold_tax_refund_if_pay_is_zero is not UNSET:
            field_dict[
                "withholdTaxRefundIfPayIsZero"
            ] = withhold_tax_refund_if_pay_is_zero
        if mileage_vehicle_type is not UNSET:
            field_dict["mileageVehicleType"] = mileage_vehicle_type
        if maps_miles is not UNSET:
            field_dict["mapsMiles"] = maps_miles
        if tax_and_ni is not UNSET:
            field_dict["taxAndNi"] = tax_and_ni
        if fps_fields is not UNSET:
            field_dict["fpsFields"] = fps_fields
        if regular_pay_lines is not UNSET:
            field_dict["regularPayLines"] = regular_pay_lines
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _period = d.pop("period", UNSET)
        period: Union[Unset, PayPeriods]
        if isinstance(_period, Unset):
            period = UNSET
        else:
            period = PayPeriods(_period)

        ordinal = d.pop("ordinal", UNSET)

        pay_amount = d.pop("payAmount", UNSET)

        _basis = d.pop("basis", UNSET)
        basis: Union[Unset, PayBasis]
        if isinstance(_basis, Unset):
            basis = UNSET
        else:
            basis = PayBasis(_basis)

        national_minimum_wage = d.pop("nationalMinimumWage", UNSET)

        pay_amount_multiplier = d.pop("payAmountMultiplier", UNSET)

        base_hourly_rate = d.pop("baseHourlyRate", UNSET)

        base_daily_rate = d.pop("baseDailyRate", UNSET)

        auto_adjust_for_leave = d.pop("autoAdjustForLeave", UNSET)

        _method = d.pop("method", UNSET)
        method: Union[Unset, PayMethod]
        if isinstance(_method, Unset):
            method = UNSET
        else:
            method = PayMethod(_method)

        pay_code = d.pop("payCode", UNSET)

        withhold_tax_refund_if_pay_is_zero = d.pop(
            "withholdTaxRefundIfPayIsZero", UNSET
        )

        _mileage_vehicle_type = d.pop("mileageVehicleType", UNSET)
        mileage_vehicle_type: Union[Unset, MileageVehicleType]
        if isinstance(_mileage_vehicle_type, Unset):
            mileage_vehicle_type = UNSET
        else:
            mileage_vehicle_type = MileageVehicleType(_mileage_vehicle_type)

        maps_miles = d.pop("mapsMiles", UNSET)

        _tax_and_ni = d.pop("taxAndNi", UNSET)
        tax_and_ni: Union[Unset, TaxAndNi]
        if isinstance(_tax_and_ni, Unset):
            tax_and_ni = UNSET
        else:
            tax_and_ni = TaxAndNi.from_dict(_tax_and_ni)

        _fps_fields = d.pop("fpsFields", UNSET)
        fps_fields: Union[Unset, FpsFields]
        if isinstance(_fps_fields, Unset):
            fps_fields = UNSET
        else:
            fps_fields = FpsFields.from_dict(_fps_fields)

        regular_pay_lines = []
        _regular_pay_lines = d.pop("regularPayLines", UNSET)
        for regular_pay_lines_item_data in _regular_pay_lines or []:
            regular_pay_lines_item = PayLine.from_dict(regular_pay_lines_item_data)

            regular_pay_lines.append(regular_pay_lines_item)

        tags = cast(List[str], d.pop("tags", UNSET))

        pay_options = cls(
            period=period,
            ordinal=ordinal,
            pay_amount=pay_amount,
            basis=basis,
            national_minimum_wage=national_minimum_wage,
            pay_amount_multiplier=pay_amount_multiplier,
            base_hourly_rate=base_hourly_rate,
            base_daily_rate=base_daily_rate,
            auto_adjust_for_leave=auto_adjust_for_leave,
            method=method,
            pay_code=pay_code,
            withhold_tax_refund_if_pay_is_zero=withhold_tax_refund_if_pay_is_zero,
            mileage_vehicle_type=mileage_vehicle_type,
            maps_miles=maps_miles,
            tax_and_ni=tax_and_ni,
            fps_fields=fps_fields,
            regular_pay_lines=regular_pay_lines,
            tags=tags,
        )

        return pay_options



from enum import Enum


class FpsLateReason(str, Enum):
    NONEGIVEN = "NoneGiven"
    NOTIONALEXPAT = "NotionalExpat"
    NOTIONALERS = "NotionalErs"
    NOTIONALOTHER = "NotionalOther"
    CLASS1 = "Class1"
    MICROEMPLOYER = "MicroEmployer"
    NOREQUIREMENT = "NoRequirement"
    REASONABLEEXCUSE = "ReasonableExcuse"
    CORRECTION = "Correction"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.papdis_payroll_period import PapdisPayrollPeriod
from ..models.pension_rule import PensionRule
from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisEmployer")


@attr.s(auto_attribs=True)
class PapdisEmployer:
    """
    Attributes:
        pension_rule (Union[Unset, PensionRule]):
        employer_id (Union[Unset, None, str]): [readonly] Taken from the papdisEmployerId property of the
            PensionProvider
        group (Union[Unset, None, str]): [readonly] Taken from the papdisGroup property of the WorkerGroup
        sub_group (Union[Unset, None, str]): [readonly] Taken from the papdisSubGroup property of the WorkerGroup
        payroll_period (Union[Unset, PapdisPayrollPeriod]):
        staging_date (Union[Unset, datetime.date]): [readonly]
        cyclical_reenrolment_date (Union[Unset, None, datetime.date]): [readonly]
    """

    pension_rule: Union[Unset, PensionRule] = UNSET
    employer_id: Union[Unset, None, str] = UNSET
    group: Union[Unset, None, str] = UNSET
    sub_group: Union[Unset, None, str] = UNSET
    payroll_period: Union[Unset, PapdisPayrollPeriod] = UNSET
    staging_date: Union[Unset, datetime.date] = UNSET
    cyclical_reenrolment_date: Union[Unset, None, datetime.date] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pension_rule: Union[Unset, str] = UNSET
        if not isinstance(self.pension_rule, Unset):
            pension_rule = self.pension_rule.value

        employer_id = self.employer_id
        group = self.group
        sub_group = self.sub_group
        payroll_period: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.payroll_period, Unset):
            payroll_period = self.payroll_period.to_dict()

        staging_date: Union[Unset, str] = UNSET
        if not isinstance(self.staging_date, Unset):
            staging_date = self.staging_date.isoformat()

        cyclical_reenrolment_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.cyclical_reenrolment_date, Unset):
            cyclical_reenrolment_date = (
                self.cyclical_reenrolment_date.isoformat()
                if self.cyclical_reenrolment_date
                else None
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pension_rule is not UNSET:
            field_dict["pensionRule"] = pension_rule
        if employer_id is not UNSET:
            field_dict["employerId"] = employer_id
        if group is not UNSET:
            field_dict["group"] = group
        if sub_group is not UNSET:
            field_dict["subGroup"] = sub_group
        if payroll_period is not UNSET:
            field_dict["payrollPeriod"] = payroll_period
        if staging_date is not UNSET:
            field_dict["stagingDate"] = staging_date
        if cyclical_reenrolment_date is not UNSET:
            field_dict["cyclicalReenrolmentDate"] = cyclical_reenrolment_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _pension_rule = d.pop("pensionRule", UNSET)
        pension_rule: Union[Unset, PensionRule]
        if isinstance(_pension_rule, Unset):
            pension_rule = UNSET
        else:
            pension_rule = PensionRule(_pension_rule)

        employer_id = d.pop("employerId", UNSET)

        group = d.pop("group", UNSET)

        sub_group = d.pop("subGroup", UNSET)

        _payroll_period = d.pop("payrollPeriod", UNSET)
        payroll_period: Union[Unset, PapdisPayrollPeriod]
        if isinstance(_payroll_period, Unset):
            payroll_period = UNSET
        else:
            payroll_period = PapdisPayrollPeriod.from_dict(_payroll_period)

        _staging_date = d.pop("stagingDate", UNSET)
        staging_date: Union[Unset, datetime.date]
        if isinstance(_staging_date, Unset):
            staging_date = UNSET
        else:
            staging_date = isoparse(_staging_date).date()

        _cyclical_reenrolment_date = d.pop("cyclicalReenrolmentDate", UNSET)
        cyclical_reenrolment_date: Union[Unset, None, datetime.date]
        if _cyclical_reenrolment_date is None:
            cyclical_reenrolment_date = None
        elif isinstance(_cyclical_reenrolment_date, Unset):
            cyclical_reenrolment_date = UNSET
        else:
            cyclical_reenrolment_date = isoparse(_cyclical_reenrolment_date).date()

        papdis_employer = cls(
            pension_rule=pension_rule,
            employer_id=employer_id,
            group=group,
            sub_group=sub_group,
            payroll_period=payroll_period,
            staging_date=staging_date,
            cyclical_reenrolment_date=cyclical_reenrolment_date,
        )

        return papdis_employer



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.p11d_single_item import P11DSingleItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DExpenses")


@attr.s(auto_attribs=True)
class P11DExpenses:
    """
    Attributes:
        trav_and_sub (Union[Unset, P11DSingleItem]):
        ent (Union[Unset, P11DSingleItem]):
        home_tel (Union[Unset, P11DSingleItem]):
        non_qual_rel (Union[Unset, P11DSingleItem]):
        other (Union[Unset, P11DSingleItem]):
        type_letter (Union[Unset, None, str]):
    """

    trav_and_sub: Union[Unset, P11DSingleItem] = UNSET
    ent: Union[Unset, P11DSingleItem] = UNSET
    home_tel: Union[Unset, P11DSingleItem] = UNSET
    non_qual_rel: Union[Unset, P11DSingleItem] = UNSET
    other: Union[Unset, P11DSingleItem] = UNSET
    type_letter: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        trav_and_sub: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trav_and_sub, Unset):
            trav_and_sub = self.trav_and_sub.to_dict()

        ent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.ent, Unset):
            ent = self.ent.to_dict()

        home_tel: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.home_tel, Unset):
            home_tel = self.home_tel.to_dict()

        non_qual_rel: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.non_qual_rel, Unset):
            non_qual_rel = self.non_qual_rel.to_dict()

        other: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.other, Unset):
            other = self.other.to_dict()

        type_letter = self.type_letter

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if trav_and_sub is not UNSET:
            field_dict["travAndSub"] = trav_and_sub
        if ent is not UNSET:
            field_dict["ent"] = ent
        if home_tel is not UNSET:
            field_dict["homeTel"] = home_tel
        if non_qual_rel is not UNSET:
            field_dict["nonQualRel"] = non_qual_rel
        if other is not UNSET:
            field_dict["other"] = other
        if type_letter is not UNSET:
            field_dict["typeLetter"] = type_letter

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _trav_and_sub = d.pop("travAndSub", UNSET)
        trav_and_sub: Union[Unset, P11DSingleItem]
        if isinstance(_trav_and_sub, Unset):
            trav_and_sub = UNSET
        else:
            trav_and_sub = P11DSingleItem.from_dict(_trav_and_sub)

        _ent = d.pop("ent", UNSET)
        ent: Union[Unset, P11DSingleItem]
        if isinstance(_ent, Unset):
            ent = UNSET
        else:
            ent = P11DSingleItem.from_dict(_ent)

        _home_tel = d.pop("homeTel", UNSET)
        home_tel: Union[Unset, P11DSingleItem]
        if isinstance(_home_tel, Unset):
            home_tel = UNSET
        else:
            home_tel = P11DSingleItem.from_dict(_home_tel)

        _non_qual_rel = d.pop("nonQualRel", UNSET)
        non_qual_rel: Union[Unset, P11DSingleItem]
        if isinstance(_non_qual_rel, Unset):
            non_qual_rel = UNSET
        else:
            non_qual_rel = P11DSingleItem.from_dict(_non_qual_rel)

        _other = d.pop("other", UNSET)
        other: Union[Unset, P11DSingleItem]
        if isinstance(_other, Unset):
            other = UNSET
        else:
            other = P11DSingleItem.from_dict(_other)

        type_letter = d.pop("typeLetter", UNSET)

        p11d_expenses = cls(
            trav_and_sub=trav_and_sub,
            ent=ent,
            home_tel=home_tel,
            non_qual_rel=non_qual_rel,
            other=other,
            type_letter=type_letter,
        )

        return p11d_expenses



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.pay_periods import PayPeriods
from ...models.pay_run import PayRun
from ...models.umbrella_payment import UmbrellaPayment
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    json_body: List[UmbrellaPayment],
    ordinal: Union[Unset, None, int] = 1,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/payrun/{payPeriod}/importumbrellapay".format(
        client.base_url, employerId=employer_id, payPeriod=pay_period
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["ordinal"] = ordinal

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = []
    for json_body_item_data in json_body:
        json_body_item = json_body_item_data.to_dict()

        json_json_body.append(json_body_item)

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[PayRun]:
    if response.status_code == 200:
        response_200 = PayRun.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[PayRun]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    json_body: List[UmbrellaPayment],
    ordinal: Union[Unset, None, int] = 1,
) -> Response[PayRun]:
    """Import Umbrella Pay

     Takes a list UmbrellaPayment and updates the currently open payrun to use the amounts given.
    You must have an open payrun for the PayPeriod specified and all payroll codes submitted must match
    an employee on the payrun.
    You should have also set the UmbrellaSettings for the Employer

    Args:
        employer_id (str):
        pay_period (PayPeriods):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (List[UmbrellaPayment]):

    Returns:
        Response[PayRun]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        pay_period=pay_period,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    json_body: List[UmbrellaPayment],
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[PayRun]:
    """Import Umbrella Pay

     Takes a list UmbrellaPayment and updates the currently open payrun to use the amounts given.
    You must have an open payrun for the PayPeriod specified and all payroll codes submitted must match
    an employee on the payrun.
    You should have also set the UmbrellaSettings for the Employer

    Args:
        employer_id (str):
        pay_period (PayPeriods):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (List[UmbrellaPayment]):

    Returns:
        Response[PayRun]
    """

    return sync_detailed(
        employer_id=employer_id,
        pay_period=pay_period,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    json_body: List[UmbrellaPayment],
    ordinal: Union[Unset, None, int] = 1,
) -> Response[PayRun]:
    """Import Umbrella Pay

     Takes a list UmbrellaPayment and updates the currently open payrun to use the amounts given.
    You must have an open payrun for the PayPeriod specified and all payroll codes submitted must match
    an employee on the payrun.
    You should have also set the UmbrellaSettings for the Employer

    Args:
        employer_id (str):
        pay_period (PayPeriods):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (List[UmbrellaPayment]):

    Returns:
        Response[PayRun]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        pay_period=pay_period,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    pay_period: PayPeriods,
    *,
    client: Client,
    json_body: List[UmbrellaPayment],
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[PayRun]:
    """Import Umbrella Pay

     Takes a list UmbrellaPayment and updates the currently open payrun to use the amounts given.
    You must have an open payrun for the PayPeriod specified and all payroll codes submitted must match
    an employee on the payrun.
    You should have also set the UmbrellaSettings for the Employer

    Args:
        employer_id (str):
        pay_period (PayPeriods):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (List[UmbrellaPayment]):

    Returns:
        Response[PayRun]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            pay_period=pay_period,
            client=client,
            json_body=json_body,
            ordinal=ordinal,
        )
    ).parsed



from typing import Any, Dict, Optional, cast

import httpx

from ...client import Client
from ...types import Response


def _get_kwargs(
    id: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/employers/{id}/suggestPayrollCode".format(client.base_url, id=id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[str]:
    if response.status_code == 200:
        response_200 = cast(str, response.json())
        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[str]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: Client,
) -> Response[str]:
    """Suggest Payroll Code

     This helper method returns a unique code for the next Employee that you create for the specified
    Employer.
    You don't have to use the value provided, you can use any value that is unqiue across Employees for
    the Employer.

    Args:
        id (str):

    Returns:
        Response[str]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    id: str,
    *,
    client: Client,
) -> Optional[str]:
    """Suggest Payroll Code

     This helper method returns a unique code for the next Employee that you create for the specified
    Employer.
    You don't have to use the value provided, you can use any value that is unqiue across Employees for
    the Employer.

    Args:
        id (str):

    Returns:
        Response[str]
    """

    return sync_detailed(
        id=id,
        client=client,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: Client,
) -> Response[str]:
    """Suggest Payroll Code

     This helper method returns a unique code for the next Employee that you create for the specified
    Employer.
    You don't have to use the value provided, you can use any value that is unqiue across Employees for
    the Employer.

    Args:
        id (str):

    Returns:
        Response[str]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    id: str,
    *,
    client: Client,
) -> Optional[str]:
    """Suggest Payroll Code

     This helper method returns a unique code for the next Employee that you create for the specified
    Employer.
    You don't have to use the value provided, you can use any value that is unqiue across Employees for
    the Employer.

    Args:
        id (str):

    Returns:
        Response[str]
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NationalInsuranceCalculationBase")


@attr.s(auto_attribs=True)
class NationalInsuranceCalculationBase:
    """Included as part of the PayRunEntry model to provide details of how the National Insurance Contribution was
    calculated.
    Unless the PayRunEntry.ManualNi property is set to true then these value will all be read-only and are recalculated
    everytime a payrun is updated.
    This calculation could be made up of one or more calculations made on different NI table letters.
    Where more than NI table letter affects the calculation, the calculation for each NI table letter will be contain in
    the Breakdown.

        Attributes:
            ni_category (Union[Unset, str]): Calculated on NI table letter
            as_director (Union[Unset, bool]): Calculated as a Director
            earnings_upto_including_lel (Union[Unset, float]): Earnings up to and including LEL
            earnings_above_lel_upto_including_pt (Union[Unset, float]): Earnings above LEL up to PT
            earnings_above_pt_upto_including_st (Union[Unset, float]): Earnings above PT up to ST
            earnings_above_pt_upto_including_uel (Union[Unset, float]): Earnings above PT up to UEL
            earnings_above_st_upto_including_uel (Union[Unset, float]): Earnings above ST up to UEL
            earnings_above_st_upto_including_fust (Union[Unset, None, float]): Earnings above ST up to FUST
            earnings_above_fust_upto_including_uel (Union[Unset, None, float]): Earnings above FUST up to UEL
            earnings_above_uel (Union[Unset, float]): Earnings above UEL
            employee_ni_gross (Union[Unset, float]): Employee National Insurance Gross Value
            employee_ni_rebate (Union[Unset, float]): Employee National Insurance Rebate Value
            employer_ni_gross (Union[Unset, float]): Employer National Insurance Gross Value
            employer_ni_rebate (Union[Unset, float]): Employer National Insurance Rebate Value
            employee_ni (Union[Unset, float]): [readonly] Net Employee National Insurance
            employer_ni (Union[Unset, float]): [readonly] Net Employer National Insurance
            net_ni (Union[Unset, float]): [readonly] Net National Insurance (Employer + Employee)
    """

    ni_category: Union[Unset, str] = UNSET
    as_director: Union[Unset, bool] = UNSET
    earnings_upto_including_lel: Union[Unset, float] = UNSET
    earnings_above_lel_upto_including_pt: Union[Unset, float] = UNSET
    earnings_above_pt_upto_including_st: Union[Unset, float] = UNSET
    earnings_above_pt_upto_including_uel: Union[Unset, float] = UNSET
    earnings_above_st_upto_including_uel: Union[Unset, float] = UNSET
    earnings_above_st_upto_including_fust: Union[Unset, None, float] = UNSET
    earnings_above_fust_upto_including_uel: Union[Unset, None, float] = UNSET
    earnings_above_uel: Union[Unset, float] = UNSET
    employee_ni_gross: Union[Unset, float] = UNSET
    employee_ni_rebate: Union[Unset, float] = UNSET
    employer_ni_gross: Union[Unset, float] = UNSET
    employer_ni_rebate: Union[Unset, float] = UNSET
    employee_ni: Union[Unset, float] = UNSET
    employer_ni: Union[Unset, float] = UNSET
    net_ni: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        ni_category = self.ni_category
        as_director = self.as_director
        earnings_upto_including_lel = self.earnings_upto_including_lel
        earnings_above_lel_upto_including_pt = self.earnings_above_lel_upto_including_pt
        earnings_above_pt_upto_including_st = self.earnings_above_pt_upto_including_st
        earnings_above_pt_upto_including_uel = self.earnings_above_pt_upto_including_uel
        earnings_above_st_upto_including_uel = self.earnings_above_st_upto_including_uel
        earnings_above_st_upto_including_fust = (
            self.earnings_above_st_upto_including_fust
        )
        earnings_above_fust_upto_including_uel = (
            self.earnings_above_fust_upto_including_uel
        )
        earnings_above_uel = self.earnings_above_uel
        employee_ni_gross = self.employee_ni_gross
        employee_ni_rebate = self.employee_ni_rebate
        employer_ni_gross = self.employer_ni_gross
        employer_ni_rebate = self.employer_ni_rebate
        employee_ni = self.employee_ni
        employer_ni = self.employer_ni
        net_ni = self.net_ni

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if ni_category is not UNSET:
            field_dict["niCategory"] = ni_category
        if as_director is not UNSET:
            field_dict["asDirector"] = as_director
        if earnings_upto_including_lel is not UNSET:
            field_dict["earningsUptoIncludingLEL"] = earnings_upto_including_lel
        if earnings_above_lel_upto_including_pt is not UNSET:
            field_dict[
                "earningsAboveLELUptoIncludingPT"
            ] = earnings_above_lel_upto_including_pt
        if earnings_above_pt_upto_including_st is not UNSET:
            field_dict[
                "earningsAbovePTUptoIncludingST"
            ] = earnings_above_pt_upto_including_st
        if earnings_above_pt_upto_including_uel is not UNSET:
            field_dict[
                "earningsAbovePTUptoIncludingUEL"
            ] = earnings_above_pt_upto_including_uel
        if earnings_above_st_upto_including_uel is not UNSET:
            field_dict[
                "earningsAboveSTUptoIncludingUEL"
            ] = earnings_above_st_upto_including_uel
        if earnings_above_st_upto_including_fust is not UNSET:
            field_dict[
                "earningsAboveSTUptoIncludingFUST"
            ] = earnings_above_st_upto_including_fust
        if earnings_above_fust_upto_including_uel is not UNSET:
            field_dict[
                "earningsAboveFUSTUptoIncludingUEL"
            ] = earnings_above_fust_upto_including_uel
        if earnings_above_uel is not UNSET:
            field_dict["earningsAboveUEL"] = earnings_above_uel
        if employee_ni_gross is not UNSET:
            field_dict["employeeNiGross"] = employee_ni_gross
        if employee_ni_rebate is not UNSET:
            field_dict["employeeNiRebate"] = employee_ni_rebate
        if employer_ni_gross is not UNSET:
            field_dict["employerNiGross"] = employer_ni_gross
        if employer_ni_rebate is not UNSET:
            field_dict["employerNiRebate"] = employer_ni_rebate
        if employee_ni is not UNSET:
            field_dict["employeeNi"] = employee_ni
        if employer_ni is not UNSET:
            field_dict["employerNi"] = employer_ni
        if net_ni is not UNSET:
            field_dict["netNi"] = net_ni

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ni_category = d.pop("niCategory", UNSET)

        as_director = d.pop("asDirector", UNSET)

        earnings_upto_including_lel = d.pop("earningsUptoIncludingLEL", UNSET)

        earnings_above_lel_upto_including_pt = d.pop(
            "earningsAboveLELUptoIncludingPT", UNSET
        )

        earnings_above_pt_upto_including_st = d.pop(
            "earningsAbovePTUptoIncludingST", UNSET
        )

        earnings_above_pt_upto_including_uel = d.pop(
            "earningsAbovePTUptoIncludingUEL", UNSET
        )

        earnings_above_st_upto_including_uel = d.pop(
            "earningsAboveSTUptoIncludingUEL", UNSET
        )

        earnings_above_st_upto_including_fust = d.pop(
            "earningsAboveSTUptoIncludingFUST", UNSET
        )

        earnings_above_fust_upto_including_uel = d.pop(
            "earningsAboveFUSTUptoIncludingUEL", UNSET
        )

        earnings_above_uel = d.pop("earningsAboveUEL", UNSET)

        employee_ni_gross = d.pop("employeeNiGross", UNSET)

        employee_ni_rebate = d.pop("employeeNiRebate", UNSET)

        employer_ni_gross = d.pop("employerNiGross", UNSET)

        employer_ni_rebate = d.pop("employerNiRebate", UNSET)

        employee_ni = d.pop("employeeNi", UNSET)

        employer_ni = d.pop("employerNi", UNSET)

        net_ni = d.pop("netNi", UNSET)

        national_insurance_calculation_base = cls(
            ni_category=ni_category,
            as_director=as_director,
            earnings_upto_including_lel=earnings_upto_including_lel,
            earnings_above_lel_upto_including_pt=earnings_above_lel_upto_including_pt,
            earnings_above_pt_upto_including_st=earnings_above_pt_upto_including_st,
            earnings_above_pt_upto_including_uel=earnings_above_pt_upto_including_uel,
            earnings_above_st_upto_including_uel=earnings_above_st_upto_including_uel,
            earnings_above_st_upto_including_fust=earnings_above_st_upto_including_fust,
            earnings_above_fust_upto_including_uel=earnings_above_fust_upto_including_uel,
            earnings_above_uel=earnings_above_uel,
            employee_ni_gross=employee_ni_gross,
            employee_ni_rebate=employee_ni_rebate,
            employer_ni_gross=employer_ni_gross,
            employer_ni_rebate=employer_ni_rebate,
            employee_ni=employee_ni,
            employer_ni=employer_ni,
            net_ni=net_ni,
        )

        return national_insurance_calculation_base



import datetime
from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.ae_assessment import AeAssessment
from ...models.ae_employee_state import AeEmployeeState
from ...models.ae_status import AeStatus
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    state: Union[Unset, None, AeEmployeeState] = UNSET,
    status: Union[Unset, None, AeStatus] = UNSET,
    state_date: Union[Unset, None, datetime.datetime] = UNSET,
    pension_id: Union[Unset, None, str] = UNSET,
    worker_group_id: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/employees/{employeeId}/autoenrolment/state".format(
        client.base_url, employerId=employer_id, employeeId=employee_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_state: Union[Unset, None, str] = UNSET
    if not isinstance(state, Unset):
        json_state = state.value if state else None

    params["state"] = json_state

    json_status: Union[Unset, None, str] = UNSET
    if not isinstance(status, Unset):
        json_status = status.value if status else None

    params["status"] = json_status

    json_state_date: Union[Unset, None, str] = UNSET
    if not isinstance(state_date, Unset):
        json_state_date = state_date.isoformat() if state_date else None

    params["stateDate"] = json_state_date

    params["pensionId"] = pension_id

    params["workerGroupId"] = worker_group_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[AeAssessment, Any]]:
    if response.status_code == 201:
        response_201 = AeAssessment.from_dict(response.json())

        return response_201
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[AeAssessment, Any]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    state: Union[Unset, None, AeEmployeeState] = UNSET,
    status: Union[Unset, None, AeStatus] = UNSET,
    state_date: Union[Unset, None, datetime.datetime] = UNSET,
    pension_id: Union[Unset, None, str] = UNSET,
    worker_group_id: Union[Unset, None, str] = UNSET,
) -> Response[Union[AeAssessment, Any]]:
    """Update State

     Updates the AutoEnrolment state for an Employee.
    You would use this API call to process notices such as Opt Out, Opt In, etc.
    A new assessment is returned showing the result of the state change.

    Args:
        employer_id (str):
        employee_id (str):
        state (Union[Unset, None, AeEmployeeState]):
        status (Union[Unset, None, AeStatus]):
        state_date (Union[Unset, None, datetime.datetime]):
        pension_id (Union[Unset, None, str]):
        worker_group_id (Union[Unset, None, str]):

    Returns:
        Response[Union[AeAssessment, Any]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        employee_id=employee_id,
        client=client,
        state=state,
        status=status,
        state_date=state_date,
        pension_id=pension_id,
        worker_group_id=worker_group_id,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    state: Union[Unset, None, AeEmployeeState] = UNSET,
    status: Union[Unset, None, AeStatus] = UNSET,
    state_date: Union[Unset, None, datetime.datetime] = UNSET,
    pension_id: Union[Unset, None, str] = UNSET,
    worker_group_id: Union[Unset, None, str] = UNSET,
) -> Optional[Union[AeAssessment, Any]]:
    """Update State

     Updates the AutoEnrolment state for an Employee.
    You would use this API call to process notices such as Opt Out, Opt In, etc.
    A new assessment is returned showing the result of the state change.

    Args:
        employer_id (str):
        employee_id (str):
        state (Union[Unset, None, AeEmployeeState]):
        status (Union[Unset, None, AeStatus]):
        state_date (Union[Unset, None, datetime.datetime]):
        pension_id (Union[Unset, None, str]):
        worker_group_id (Union[Unset, None, str]):

    Returns:
        Response[Union[AeAssessment, Any]]
    """

    return sync_detailed(
        employer_id=employer_id,
        employee_id=employee_id,
        client=client,
        state=state,
        status=status,
        state_date=state_date,
        pension_id=pension_id,
        worker_group_id=worker_group_id,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    state: Union[Unset, None, AeEmployeeState] = UNSET,
    status: Union[Unset, None, AeStatus] = UNSET,
    state_date: Union[Unset, None, datetime.datetime] = UNSET,
    pension_id: Union[Unset, None, str] = UNSET,
    worker_group_id: Union[Unset, None, str] = UNSET,
) -> Response[Union[AeAssessment, Any]]:
    """Update State

     Updates the AutoEnrolment state for an Employee.
    You would use this API call to process notices such as Opt Out, Opt In, etc.
    A new assessment is returned showing the result of the state change.

    Args:
        employer_id (str):
        employee_id (str):
        state (Union[Unset, None, AeEmployeeState]):
        status (Union[Unset, None, AeStatus]):
        state_date (Union[Unset, None, datetime.datetime]):
        pension_id (Union[Unset, None, str]):
        worker_group_id (Union[Unset, None, str]):

    Returns:
        Response[Union[AeAssessment, Any]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        employee_id=employee_id,
        client=client,
        state=state,
        status=status,
        state_date=state_date,
        pension_id=pension_id,
        worker_group_id=worker_group_id,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    state: Union[Unset, None, AeEmployeeState] = UNSET,
    status: Union[Unset, None, AeStatus] = UNSET,
    state_date: Union[Unset, None, datetime.datetime] = UNSET,
    pension_id: Union[Unset, None, str] = UNSET,
    worker_group_id: Union[Unset, None, str] = UNSET,
) -> Optional[Union[AeAssessment, Any]]:
    """Update State

     Updates the AutoEnrolment state for an Employee.
    You would use this API call to process notices such as Opt Out, Opt In, etc.
    A new assessment is returned showing the result of the state change.

    Args:
        employer_id (str):
        employee_id (str):
        state (Union[Unset, None, AeEmployeeState]):
        status (Union[Unset, None, AeStatus]):
        state_date (Union[Unset, None, datetime.datetime]):
        pension_id (Union[Unset, None, str]):
        worker_group_id (Union[Unset, None, str]):

    Returns:
        Response[Union[AeAssessment, Any]]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            employee_id=employee_id,
            client=client,
            state=state,
            status=status,
            state_date=state_date,
            pension_id=pension_id,
            worker_group_id=worker_group_id,
        )
    ).parsed



from enum import Enum


class PayMethod(str, Enum):
    CASH = "Cash"
    CHEQUE = "Cheque"
    CREDIT = "Credit"
    DIRECTDEBIT = "DirectDebit"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.address import Address
from ..types import UNSET, Unset

T = TypeVar("T", bound="PensionAdministrator")


@attr.s(auto_attribs=True)
class PensionAdministrator:
    """
    Attributes:
        name (Union[Unset, None, str]):
        email (Union[Unset, None, str]):
        address (Union[Unset, Address]):
        telephone (Union[Unset, None, str]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    name: Union[Unset, None, str] = UNSET
    email: Union[Unset, None, str] = UNSET
    address: Union[Unset, Address] = UNSET
    telephone: Union[Unset, None, str] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        email = self.email
        address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        telephone = self.telephone
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if email is not UNSET:
            field_dict["email"] = email
        if address is not UNSET:
            field_dict["address"] = address
        if telephone is not UNSET:
            field_dict["telephone"] = telephone
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        email = d.pop("email", UNSET)

        _address = d.pop("address", UNSET)
        address: Union[Unset, Address]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = Address.from_dict(_address)

        telephone = d.pop("telephone", UNSET)

        id = d.pop("id", UNSET)

        pension_administrator = cls(
            name=name,
            email=email,
            address=address,
            telephone=telephone,
            id=id,
        )

        return pension_administrator



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ExbP11DbClass1AAdjustment")


@attr.s(auto_attribs=True)
class ExbP11DbClass1AAdjustment:
    """
    Attributes:
        description (Union[Unset, None, str]):
        adjustment (Union[Unset, None, str]):
    """

    description: Union[Unset, None, str] = UNSET
    adjustment: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        description = self.description
        adjustment = self.adjustment

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if adjustment is not UNSET:
            field_dict["adjustment"] = adjustment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        description = d.pop("description", UNSET)

        adjustment = d.pop("adjustment", UNSET)

        exb_p11_db_class_1a_adjustment = cls(
            description=description,
            adjustment=adjustment,
        )

        return exb_p11_db_class_1a_adjustment



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.auth_scheme import AuthScheme
from ..models.external_data_provider_id import ExternalDataProviderId
from ..models.external_data_provider_type import ExternalDataProviderType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExternalDataProvider")


@attr.s(auto_attribs=True)
class ExternalDataProvider:
    """
    Attributes:
        name (Union[Unset, None, str]):
        id (Union[Unset, ExternalDataProviderId]):
        auth_scheme (Union[Unset, AuthScheme]):
        logo_url (Union[Unset, None, str]):
        icon_url (Union[Unset, None, str]):
        connected (Union[Unset, bool]):
        connected_as (Union[Unset, None, str]):
        requires_config (Union[Unset, bool]):
        types (Union[Unset, None, List[ExternalDataProviderType]]):
    """

    name: Union[Unset, None, str] = UNSET
    id: Union[Unset, ExternalDataProviderId] = UNSET
    auth_scheme: Union[Unset, AuthScheme] = UNSET
    logo_url: Union[Unset, None, str] = UNSET
    icon_url: Union[Unset, None, str] = UNSET
    connected: Union[Unset, bool] = UNSET
    connected_as: Union[Unset, None, str] = UNSET
    requires_config: Union[Unset, bool] = UNSET
    types: Union[Unset, None, List[ExternalDataProviderType]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        id: Union[Unset, str] = UNSET
        if not isinstance(self.id, Unset):
            id = self.id.value

        auth_scheme: Union[Unset, str] = UNSET
        if not isinstance(self.auth_scheme, Unset):
            auth_scheme = self.auth_scheme.value

        logo_url = self.logo_url
        icon_url = self.icon_url
        connected = self.connected
        connected_as = self.connected_as
        requires_config = self.requires_config
        types: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.types, Unset):
            if self.types is None:
                types = None
            else:
                types = []
                for types_item_data in self.types:
                    types_item = types_item_data.value

                    types.append(types_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if id is not UNSET:
            field_dict["id"] = id
        if auth_scheme is not UNSET:
            field_dict["authScheme"] = auth_scheme
        if logo_url is not UNSET:
            field_dict["logoUrl"] = logo_url
        if icon_url is not UNSET:
            field_dict["iconUrl"] = icon_url
        if connected is not UNSET:
            field_dict["connected"] = connected
        if connected_as is not UNSET:
            field_dict["connectedAs"] = connected_as
        if requires_config is not UNSET:
            field_dict["requiresConfig"] = requires_config
        if types is not UNSET:
            field_dict["types"] = types

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        _id = d.pop("id", UNSET)
        id: Union[Unset, ExternalDataProviderId]
        if isinstance(_id, Unset):
            id = UNSET
        else:
            id = ExternalDataProviderId(_id)

        _auth_scheme = d.pop("authScheme", UNSET)
        auth_scheme: Union[Unset, AuthScheme]
        if isinstance(_auth_scheme, Unset):
            auth_scheme = UNSET
        else:
            auth_scheme = AuthScheme(_auth_scheme)

        logo_url = d.pop("logoUrl", UNSET)

        icon_url = d.pop("iconUrl", UNSET)

        connected = d.pop("connected", UNSET)

        connected_as = d.pop("connectedAs", UNSET)

        requires_config = d.pop("requiresConfig", UNSET)

        types = []
        _types = d.pop("types", UNSET)
        for types_item_data in _types or []:
            types_item = ExternalDataProviderType(types_item_data)

            types.append(types_item)

        external_data_provider = cls(
            name=name,
            id=id,
            auth_scheme=auth_scheme,
            logo_url=logo_url,
            icon_url=icon_url,
            connected=connected,
            connected_as=connected_as,
            requires_config=requires_config,
            types=types,
        )

        return external_data_provider



from enum import Enum


class PayPeriods(str, Enum):
    CUSTOM = "Custom"
    MONTHLY = "Monthly"
    FOURWEEKLY = "FourWeekly"
    FORTNIGHTLY = "Fortnightly"
    WEEKLY = "Weekly"
    DAILY = "Daily"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="P11NiValues")


@attr.s(auto_attribs=True)
class P11NiValues:
    """Forms a part of the P11 report.

    Attributes:
        table (Union[Unset, None, str]): [readonly]
        period (Union[Unset, float]): [readonly]
        ytd (Union[Unset, float]): [readonly]
    """

    table: Union[Unset, None, str] = UNSET
    period: Union[Unset, float] = UNSET
    ytd: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        table = self.table
        period = self.period
        ytd = self.ytd

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if table is not UNSET:
            field_dict["table"] = table
        if period is not UNSET:
            field_dict["period"] = period
        if ytd is not UNSET:
            field_dict["ytd"] = ytd

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        table = d.pop("table", UNSET)

        period = d.pop("period", UNSET)

        ytd = d.pop("ytd", UNSET)

        p11_ni_values = cls(
            table=table,
            period=period,
            ytd=ytd,
        )

        return p11_ni_values



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.cis_sub_contractor_type import CISSubContractorType
from ..models.cis_tax_status import CISTaxStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="CisSubContractorSummary")


@attr.s(auto_attribs=True)
class CisSubContractorSummary:
    """
    Attributes:
        payroll_code (Union[Unset, None, str]):
        trading_name (Union[Unset, None, str]):
        first_name (Union[Unset, None, str]):
        last_name (Union[Unset, None, str]):
        ni_number (Union[Unset, None, str]):
        type (Union[Unset, CISSubContractorType]):
        utr (Union[Unset, None, str]):
        tax_status (Union[Unset, CISTaxStatus]):
        verification_number (Union[Unset, None, str]):
        display_name (Union[Unset, None, str]):
        validation_message (Union[Unset, None, List[str]]):
    """

    payroll_code: Union[Unset, None, str] = UNSET
    trading_name: Union[Unset, None, str] = UNSET
    first_name: Union[Unset, None, str] = UNSET
    last_name: Union[Unset, None, str] = UNSET
    ni_number: Union[Unset, None, str] = UNSET
    type: Union[Unset, CISSubContractorType] = UNSET
    utr: Union[Unset, None, str] = UNSET
    tax_status: Union[Unset, CISTaxStatus] = UNSET
    verification_number: Union[Unset, None, str] = UNSET
    display_name: Union[Unset, None, str] = UNSET
    validation_message: Union[Unset, None, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        payroll_code = self.payroll_code
        trading_name = self.trading_name
        first_name = self.first_name
        last_name = self.last_name
        ni_number = self.ni_number
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        utr = self.utr
        tax_status: Union[Unset, str] = UNSET
        if not isinstance(self.tax_status, Unset):
            tax_status = self.tax_status.value

        verification_number = self.verification_number
        display_name = self.display_name
        validation_message: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.validation_message, Unset):
            if self.validation_message is None:
                validation_message = None
            else:
                validation_message = self.validation_message

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if trading_name is not UNSET:
            field_dict["tradingName"] = trading_name
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if ni_number is not UNSET:
            field_dict["niNumber"] = ni_number
        if type is not UNSET:
            field_dict["type"] = type
        if utr is not UNSET:
            field_dict["utr"] = utr
        if tax_status is not UNSET:
            field_dict["taxStatus"] = tax_status
        if verification_number is not UNSET:
            field_dict["verificationNumber"] = verification_number
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if validation_message is not UNSET:
            field_dict["validationMessage"] = validation_message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        payroll_code = d.pop("payrollCode", UNSET)

        trading_name = d.pop("tradingName", UNSET)

        first_name = d.pop("firstName", UNSET)

        last_name = d.pop("lastName", UNSET)

        ni_number = d.pop("niNumber", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, CISSubContractorType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = CISSubContractorType(_type)

        utr = d.pop("utr", UNSET)

        _tax_status = d.pop("taxStatus", UNSET)
        tax_status: Union[Unset, CISTaxStatus]
        if isinstance(_tax_status, Unset):
            tax_status = UNSET
        else:
            tax_status = CISTaxStatus(_tax_status)

        verification_number = d.pop("verificationNumber", UNSET)

        display_name = d.pop("displayName", UNSET)

        validation_message = cast(List[str], d.pop("validationMessage", UNSET))

        cis_sub_contractor_summary = cls(
            payroll_code=payroll_code,
            trading_name=trading_name,
            first_name=first_name,
            last_name=last_name,
            ni_number=ni_number,
            type=type,
            utr=utr,
            tax_status=tax_status,
            verification_number=verification_number,
            display_name=display_name,
            validation_message=validation_message,
        )

        return cis_sub_contractor_summary



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="RtiContact")


@attr.s(auto_attribs=True)
class RtiContact:
    """
    Attributes:
        first_name (Union[Unset, None, str]):
        last_name (Union[Unset, None, str]):
        email (Union[Unset, None, str]):
        telephone (Union[Unset, None, str]):
    """

    first_name: Union[Unset, None, str] = UNSET
    last_name: Union[Unset, None, str] = UNSET
    email: Union[Unset, None, str] = UNSET
    telephone: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        first_name = self.first_name
        last_name = self.last_name
        email = self.email
        telephone = self.telephone

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if email is not UNSET:
            field_dict["email"] = email
        if telephone is not UNSET:
            field_dict["telephone"] = telephone

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        first_name = d.pop("firstName", UNSET)

        last_name = d.pop("lastName", UNSET)

        email = d.pop("email", UNSET)

        telephone = d.pop("telephone", UNSET)

        rti_contact = cls(
            first_name=first_name,
            last_name=last_name,
            email=email,
            telephone=telephone,
        )

        return rti_contact



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.p11d_loan import P11DLoan
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DLoanCollection")


@attr.s(auto_attribs=True)
class P11DLoanCollection:
    """
    Attributes:
        loan (Union[Unset, None, List[P11DLoan]]):
        type_letter (Union[Unset, None, str]):
    """

    loan: Union[Unset, None, List[P11DLoan]] = UNSET
    type_letter: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        loan: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.loan, Unset):
            if self.loan is None:
                loan = None
            else:
                loan = []
                for loan_item_data in self.loan:
                    loan_item = loan_item_data.to_dict()

                    loan.append(loan_item)

        type_letter = self.type_letter

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if loan is not UNSET:
            field_dict["loan"] = loan
        if type_letter is not UNSET:
            field_dict["typeLetter"] = type_letter

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        loan = []
        _loan = d.pop("loan", UNSET)
        for loan_item_data in _loan or []:
            loan_item = P11DLoan.from_dict(loan_item_data)

            loan.append(loan_item)

        type_letter = d.pop("typeLetter", UNSET)

        p11d_loan_collection = cls(
            loan=loan,
            type_letter=type_letter,
        )

        return p11d_loan_collection



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.benefit_details_car_power_type import BenefitDetailsCarPowerType
from ..types import UNSET, Unset

T = TypeVar("T", bound="BenefitDetailsCar")


@attr.s(auto_attribs=True)
class BenefitDetailsCar:
    """
    Attributes:
        make_and_model (Union[Unset, None, str]):
        registration (Union[Unset, None, str]):
        first_registered (Union[Unset, None, datetime.date]):
        has_approved_emissions_value (Union[Unset, bool]):
        co_2_emissions (Union[Unset, int]):
        engine_size (Union[Unset, int]):
        zero_emissions_mileage (Union[Unset, int]):
        fuel_type (Union[Unset, BenefitDetailsCarPowerType]):
        available_from (Union[Unset, None, datetime.date]):
        available_to (Union[Unset, None, datetime.date]):
        days_unavailable (Union[Unset, int]):
        list_price (Union[Unset, float]):
        non_standard_accessories (Union[Unset, float]):
        employee_capital_contributions (Union[Unset, float]):
        employee_private_contributions (Union[Unset, float]):
        free_fuel (Union[Unset, bool]):
        fuel_available_from (Union[Unset, None, datetime.date]):
        fuel_available_to (Union[Unset, None, datetime.date]):
        free_fuel_reinstated (Union[Unset, bool]):
        registered_prior_to_1998 (Union[Unset, bool]): [readonly]
        rate (Union[Unset, float]): [readonly] The applicable rate based on CO2Emissions and Engine Size
        chargeable_value (Union[Unset, float]): [readonly] The chargeable value of the car
        full_year_charge (Union[Unset, float]): [readonly] The charge for the car for a full year, not taking in to
            account available dates or EmployeePrivateContributions
        cash_equivalent (Union[Unset, float]): [readonly]
        cash_equivalent_fuel (Union[Unset, float]): [readonly]
    """

    make_and_model: Union[Unset, None, str] = UNSET
    registration: Union[Unset, None, str] = UNSET
    first_registered: Union[Unset, None, datetime.date] = UNSET
    has_approved_emissions_value: Union[Unset, bool] = UNSET
    co_2_emissions: Union[Unset, int] = UNSET
    engine_size: Union[Unset, int] = UNSET
    zero_emissions_mileage: Union[Unset, int] = UNSET
    fuel_type: Union[Unset, BenefitDetailsCarPowerType] = UNSET
    available_from: Union[Unset, None, datetime.date] = UNSET
    available_to: Union[Unset, None, datetime.date] = UNSET
    days_unavailable: Union[Unset, int] = UNSET
    list_price: Union[Unset, float] = UNSET
    non_standard_accessories: Union[Unset, float] = UNSET
    employee_capital_contributions: Union[Unset, float] = UNSET
    employee_private_contributions: Union[Unset, float] = UNSET
    free_fuel: Union[Unset, bool] = UNSET
    fuel_available_from: Union[Unset, None, datetime.date] = UNSET
    fuel_available_to: Union[Unset, None, datetime.date] = UNSET
    free_fuel_reinstated: Union[Unset, bool] = UNSET
    registered_prior_to_1998: Union[Unset, bool] = UNSET
    rate: Union[Unset, float] = UNSET
    chargeable_value: Union[Unset, float] = UNSET
    full_year_charge: Union[Unset, float] = UNSET
    cash_equivalent: Union[Unset, float] = UNSET
    cash_equivalent_fuel: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        make_and_model = self.make_and_model
        registration = self.registration
        first_registered: Union[Unset, None, str] = UNSET
        if not isinstance(self.first_registered, Unset):
            first_registered = (
                self.first_registered.isoformat() if self.first_registered else None
            )

        has_approved_emissions_value = self.has_approved_emissions_value
        co_2_emissions = self.co_2_emissions
        engine_size = self.engine_size
        zero_emissions_mileage = self.zero_emissions_mileage
        fuel_type: Union[Unset, str] = UNSET
        if not isinstance(self.fuel_type, Unset):
            fuel_type = self.fuel_type.value

        available_from: Union[Unset, None, str] = UNSET
        if not isinstance(self.available_from, Unset):
            available_from = (
                self.available_from.isoformat() if self.available_from else None
            )

        available_to: Union[Unset, None, str] = UNSET
        if not isinstance(self.available_to, Unset):
            available_to = self.available_to.isoformat() if self.available_to else None

        days_unavailable = self.days_unavailable
        list_price = self.list_price
        non_standard_accessories = self.non_standard_accessories
        employee_capital_contributions = self.employee_capital_contributions
        employee_private_contributions = self.employee_private_contributions
        free_fuel = self.free_fuel
        fuel_available_from: Union[Unset, None, str] = UNSET
        if not isinstance(self.fuel_available_from, Unset):
            fuel_available_from = (
                self.fuel_available_from.isoformat()
                if self.fuel_available_from
                else None
            )

        fuel_available_to: Union[Unset, None, str] = UNSET
        if not isinstance(self.fuel_available_to, Unset):
            fuel_available_to = (
                self.fuel_available_to.isoformat() if self.fuel_available_to else None
            )

        free_fuel_reinstated = self.free_fuel_reinstated
        registered_prior_to_1998 = self.registered_prior_to_1998
        rate = self.rate
        chargeable_value = self.chargeable_value
        full_year_charge = self.full_year_charge
        cash_equivalent = self.cash_equivalent
        cash_equivalent_fuel = self.cash_equivalent_fuel

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if make_and_model is not UNSET:
            field_dict["makeAndModel"] = make_and_model
        if registration is not UNSET:
            field_dict["registration"] = registration
        if first_registered is not UNSET:
            field_dict["firstRegistered"] = first_registered
        if has_approved_emissions_value is not UNSET:
            field_dict["hasApprovedEmissionsValue"] = has_approved_emissions_value
        if co_2_emissions is not UNSET:
            field_dict["co2Emissions"] = co_2_emissions
        if engine_size is not UNSET:
            field_dict["engineSize"] = engine_size
        if zero_emissions_mileage is not UNSET:
            field_dict["zeroEmissionsMileage"] = zero_emissions_mileage
        if fuel_type is not UNSET:
            field_dict["fuelType"] = fuel_type
        if available_from is not UNSET:
            field_dict["availableFrom"] = available_from
        if available_to is not UNSET:
            field_dict["availableTo"] = available_to
        if days_unavailable is not UNSET:
            field_dict["daysUnavailable"] = days_unavailable
        if list_price is not UNSET:
            field_dict["listPrice"] = list_price
        if non_standard_accessories is not UNSET:
            field_dict["nonStandardAccessories"] = non_standard_accessories
        if employee_capital_contributions is not UNSET:
            field_dict["employeeCapitalContributions"] = employee_capital_contributions
        if employee_private_contributions is not UNSET:
            field_dict["employeePrivateContributions"] = employee_private_contributions
        if free_fuel is not UNSET:
            field_dict["freeFuel"] = free_fuel
        if fuel_available_from is not UNSET:
            field_dict["fuelAvailableFrom"] = fuel_available_from
        if fuel_available_to is not UNSET:
            field_dict["fuelAvailableTo"] = fuel_available_to
        if free_fuel_reinstated is not UNSET:
            field_dict["freeFuelReinstated"] = free_fuel_reinstated
        if registered_prior_to_1998 is not UNSET:
            field_dict["registeredPriorTo1998"] = registered_prior_to_1998
        if rate is not UNSET:
            field_dict["rate"] = rate
        if chargeable_value is not UNSET:
            field_dict["chargeableValue"] = chargeable_value
        if full_year_charge is not UNSET:
            field_dict["fullYearCharge"] = full_year_charge
        if cash_equivalent is not UNSET:
            field_dict["cashEquivalent"] = cash_equivalent
        if cash_equivalent_fuel is not UNSET:
            field_dict["cashEquivalentFuel"] = cash_equivalent_fuel

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        make_and_model = d.pop("makeAndModel", UNSET)

        registration = d.pop("registration", UNSET)

        _first_registered = d.pop("firstRegistered", UNSET)
        first_registered: Union[Unset, None, datetime.date]
        if _first_registered is None:
            first_registered = None
        elif isinstance(_first_registered, Unset):
            first_registered = UNSET
        else:
            first_registered = isoparse(_first_registered).date()

        has_approved_emissions_value = d.pop("hasApprovedEmissionsValue", UNSET)

        co_2_emissions = d.pop("co2Emissions", UNSET)

        engine_size = d.pop("engineSize", UNSET)

        zero_emissions_mileage = d.pop("zeroEmissionsMileage", UNSET)

        _fuel_type = d.pop("fuelType", UNSET)
        fuel_type: Union[Unset, BenefitDetailsCarPowerType]
        if isinstance(_fuel_type, Unset):
            fuel_type = UNSET
        else:
            fuel_type = BenefitDetailsCarPowerType(_fuel_type)

        _available_from = d.pop("availableFrom", UNSET)
        available_from: Union[Unset, None, datetime.date]
        if _available_from is None:
            available_from = None
        elif isinstance(_available_from, Unset):
            available_from = UNSET
        else:
            available_from = isoparse(_available_from).date()

        _available_to = d.pop("availableTo", UNSET)
        available_to: Union[Unset, None, datetime.date]
        if _available_to is None:
            available_to = None
        elif isinstance(_available_to, Unset):
            available_to = UNSET
        else:
            available_to = isoparse(_available_to).date()

        days_unavailable = d.pop("daysUnavailable", UNSET)

        list_price = d.pop("listPrice", UNSET)

        non_standard_accessories = d.pop("nonStandardAccessories", UNSET)

        employee_capital_contributions = d.pop("employeeCapitalContributions", UNSET)

        employee_private_contributions = d.pop("employeePrivateContributions", UNSET)

        free_fuel = d.pop("freeFuel", UNSET)

        _fuel_available_from = d.pop("fuelAvailableFrom", UNSET)
        fuel_available_from: Union[Unset, None, datetime.date]
        if _fuel_available_from is None:
            fuel_available_from = None
        elif isinstance(_fuel_available_from, Unset):
            fuel_available_from = UNSET
        else:
            fuel_available_from = isoparse(_fuel_available_from).date()

        _fuel_available_to = d.pop("fuelAvailableTo", UNSET)
        fuel_available_to: Union[Unset, None, datetime.date]
        if _fuel_available_to is None:
            fuel_available_to = None
        elif isinstance(_fuel_available_to, Unset):
            fuel_available_to = UNSET
        else:
            fuel_available_to = isoparse(_fuel_available_to).date()

        free_fuel_reinstated = d.pop("freeFuelReinstated", UNSET)

        registered_prior_to_1998 = d.pop("registeredPriorTo1998", UNSET)

        rate = d.pop("rate", UNSET)

        chargeable_value = d.pop("chargeableValue", UNSET)

        full_year_charge = d.pop("fullYearCharge", UNSET)

        cash_equivalent = d.pop("cashEquivalent", UNSET)

        cash_equivalent_fuel = d.pop("cashEquivalentFuel", UNSET)

        benefit_details_car = cls(
            make_and_model=make_and_model,
            registration=registration,
            first_registered=first_registered,
            has_approved_emissions_value=has_approved_emissions_value,
            co_2_emissions=co_2_emissions,
            engine_size=engine_size,
            zero_emissions_mileage=zero_emissions_mileage,
            fuel_type=fuel_type,
            available_from=available_from,
            available_to=available_to,
            days_unavailable=days_unavailable,
            list_price=list_price,
            non_standard_accessories=non_standard_accessories,
            employee_capital_contributions=employee_capital_contributions,
            employee_private_contributions=employee_private_contributions,
            free_fuel=free_fuel,
            fuel_available_from=fuel_available_from,
            fuel_available_to=fuel_available_to,
            free_fuel_reinstated=free_fuel_reinstated,
            registered_prior_to_1998=registered_prior_to_1998,
            rate=rate,
            chargeable_value=chargeable_value,
            full_year_charge=full_year_charge,
            cash_equivalent=cash_equivalent,
            cash_equivalent_fuel=cash_equivalent_fuel,
        )

        return benefit_details_car



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.mileage_vehicle_type import MileageVehicleType
from ..types import UNSET, Unset

T = TypeVar("T", bound="MileageAllowancePaymentsRate")


@attr.s(auto_attribs=True)
class MileageAllowancePaymentsRate:
    """Part of the TaxYearConfig that our engine uses to calculate Mileage Allowance Payments.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            vehicle_type (Union[Unset, MileageVehicleType]):
            rate (Union[Unset, float]):
            threshold (Union[Unset, None, int]):
            above_threshold_rate (Union[Unset, float]):
    """

    vehicle_type: Union[Unset, MileageVehicleType] = UNSET
    rate: Union[Unset, float] = UNSET
    threshold: Union[Unset, None, int] = UNSET
    above_threshold_rate: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        vehicle_type: Union[Unset, str] = UNSET
        if not isinstance(self.vehicle_type, Unset):
            vehicle_type = self.vehicle_type.value

        rate = self.rate
        threshold = self.threshold
        above_threshold_rate = self.above_threshold_rate

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if vehicle_type is not UNSET:
            field_dict["vehicleType"] = vehicle_type
        if rate is not UNSET:
            field_dict["rate"] = rate
        if threshold is not UNSET:
            field_dict["threshold"] = threshold
        if above_threshold_rate is not UNSET:
            field_dict["aboveThresholdRate"] = above_threshold_rate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _vehicle_type = d.pop("vehicleType", UNSET)
        vehicle_type: Union[Unset, MileageVehicleType]
        if isinstance(_vehicle_type, Unset):
            vehicle_type = UNSET
        else:
            vehicle_type = MileageVehicleType(_vehicle_type)

        rate = d.pop("rate", UNSET)

        threshold = d.pop("threshold", UNSET)

        above_threshold_rate = d.pop("aboveThresholdRate", UNSET)

        mileage_allowance_payments_rate = cls(
            vehicle_type=vehicle_type,
            rate=rate,
            threshold=threshold,
            above_threshold_rate=above_threshold_rate,
        )

        return mileage_allowance_payments_rate



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.overseas_employer_details import OverseasEmployerDetails
from ..models.pensioner_payroll import PensionerPayroll
from ..models.starter_declaration import StarterDeclaration
from ..types import UNSET, Unset

T = TypeVar("T", bound="StarterDetails")


@attr.s(auto_attribs=True)
class StarterDetails:
    """
    Attributes:
        start_date (datetime.date):
        starter_declaration (StarterDeclaration):
        overseas_employer_details (Union[Unset, OverseasEmployerDetails]):
        pensioner_payroll (Union[Unset, PensionerPayroll]):
    """

    start_date: datetime.date
    starter_declaration: StarterDeclaration
    overseas_employer_details: Union[Unset, OverseasEmployerDetails] = UNSET
    pensioner_payroll: Union[Unset, PensionerPayroll] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        start_date = self.start_date.isoformat()
        starter_declaration = self.starter_declaration.value

        overseas_employer_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.overseas_employer_details, Unset):
            overseas_employer_details = self.overseas_employer_details.to_dict()

        pensioner_payroll: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pensioner_payroll, Unset):
            pensioner_payroll = self.pensioner_payroll.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "startDate": start_date,
                "starterDeclaration": starter_declaration,
            }
        )
        if overseas_employer_details is not UNSET:
            field_dict["overseasEmployerDetails"] = overseas_employer_details
        if pensioner_payroll is not UNSET:
            field_dict["pensionerPayroll"] = pensioner_payroll

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start_date = isoparse(d.pop("startDate")).date()

        starter_declaration = StarterDeclaration(d.pop("starterDeclaration"))

        _overseas_employer_details = d.pop("overseasEmployerDetails", UNSET)
        overseas_employer_details: Union[Unset, OverseasEmployerDetails]
        if isinstance(_overseas_employer_details, Unset):
            overseas_employer_details = UNSET
        else:
            overseas_employer_details = OverseasEmployerDetails.from_dict(
                _overseas_employer_details
            )

        _pensioner_payroll = d.pop("pensionerPayroll", UNSET)
        pensioner_payroll: Union[Unset, PensionerPayroll]
        if isinstance(_pensioner_payroll, Unset):
            pensioner_payroll = UNSET
        else:
            pensioner_payroll = PensionerPayroll.from_dict(_pensioner_payroll)

        starter_details = cls(
            start_date=start_date,
            starter_declaration=starter_declaration,
            overseas_employer_details=overseas_employer_details,
            pensioner_payroll=pensioner_payroll,
        )

        return starter_details



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.report import Report
from ..models.tax_code_change_values import TaxCodeChangeValues
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="TaxCodeChangeReport")


@attr.s(auto_attribs=True)
class TaxCodeChangeReport:
    """
    Attributes:
        lines (Union[Unset, None, List[TaxCodeChangeValues]]):
        report (Union[Unset, Report]):
        tax_year (Union[Unset, TaxYear]):
    """

    lines: Union[Unset, None, List[TaxCodeChangeValues]] = UNSET
    report: Union[Unset, Report] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.lines, Unset):
            if self.lines is None:
                lines = None
            else:
                lines = []
                for lines_item_data in self.lines:
                    lines_item = lines_item_data.to_dict()

                    lines.append(lines_item)

        report: Union[Unset, str] = UNSET
        if not isinstance(self.report, Unset):
            report = self.report.value

        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if lines is not UNSET:
            field_dict["lines"] = lines
        if report is not UNSET:
            field_dict["report"] = report
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        lines = []
        _lines = d.pop("lines", UNSET)
        for lines_item_data in _lines or []:
            lines_item = TaxCodeChangeValues.from_dict(lines_item_data)

            lines.append(lines_item)

        _report = d.pop("report", UNSET)
        report: Union[Unset, Report]
        if isinstance(_report, Unset):
            report = UNSET
        else:
            report = Report(_report)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        tax_code_change_report = cls(
            lines=lines,
            report=report,
            tax_year=tax_year,
        )

        return tax_code_change_report



import datetime
from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.pay_periods import PayPeriods
from ...models.pay_run import PayRun
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    json_body: datetime.datetime,
    ordinal: Union[Unset, None, int] = 1,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/payrun/{taxYear}/{payPeriod}/{periodNumber}/paymentdate".format(
        client.base_url,
        employerId=employer_id,
        taxYear=tax_year,
        payPeriod=pay_period,
        periodNumber=period_number,
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["ordinal"] = ordinal

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.isoformat()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[PayRun]:
    if response.status_code == 200:
        response_200 = PayRun.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[PayRun]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    json_body: datetime.datetime,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[PayRun]:
    """Set PaymentDate

     Sets the PaymentDate for a PayRun.
    If the date given is outside of the TaxPeriod that the payrun is for then only the payment date for
    the individual
    PayRunEntries is updated, the PayRun will retain it's current PaymentDate

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (datetime.datetime):

    Returns:
        Response[PayRun]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    json_body: datetime.datetime,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[PayRun]:
    """Set PaymentDate

     Sets the PaymentDate for a PayRun.
    If the date given is outside of the TaxPeriod that the payrun is for then only the payment date for
    the individual
    PayRunEntries is updated, the PayRun will retain it's current PaymentDate

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (datetime.datetime):

    Returns:
        Response[PayRun]
    """

    return sync_detailed(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    json_body: datetime.datetime,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[PayRun]:
    """Set PaymentDate

     Sets the PaymentDate for a PayRun.
    If the date given is outside of the TaxPeriod that the payrun is for then only the payment date for
    the individual
    PayRunEntries is updated, the PayRun will retain it's current PaymentDate

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (datetime.datetime):

    Returns:
        Response[PayRun]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    json_body: datetime.datetime,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[PayRun]:
    """Set PaymentDate

     Sets the PaymentDate for a PayRun.
    If the date given is outside of the TaxPeriod that the payrun is for then only the payment date for
    the individual
    PayRunEntries is updated, the PayRun will retain it's current PaymentDate

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (datetime.datetime):

    Returns:
        Response[PayRun]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            tax_year=tax_year,
            pay_period=pay_period,
            period_number=period_number,
            client=client,
            json_body=json_body,
            ordinal=ordinal,
        )
    ).parsed



from enum import Enum


class Country(str, Enum):
    ENGLAND = "England"
    NORTHERNIRELAND = "NorthernIreland"
    SCOTLAND = "Scotland"
    WALES = "Wales"
    OUTSIDEUK = "OutsideUk"
    UK = "Uk"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.country import Country
from ..types import UNSET, Unset

T = TypeVar("T", bound="Address")


@attr.s(auto_attribs=True)
class Address:
    """
    Attributes:
        line1 (Union[Unset, None, str]):
        line2 (Union[Unset, None, str]):
        line3 (Union[Unset, None, str]):
        line4 (Union[Unset, None, str]):
        line5 (Union[Unset, None, str]):
        post_code (Union[Unset, None, str]):
        country (Union[Unset, Country]):
        foreign_country (Union[Unset, None, str]):
    """

    line1: Union[Unset, None, str] = UNSET
    line2: Union[Unset, None, str] = UNSET
    line3: Union[Unset, None, str] = UNSET
    line4: Union[Unset, None, str] = UNSET
    line5: Union[Unset, None, str] = UNSET
    post_code: Union[Unset, None, str] = UNSET
    country: Union[Unset, Country] = UNSET
    foreign_country: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        line1 = self.line1
        line2 = self.line2
        line3 = self.line3
        line4 = self.line4
        line5 = self.line5
        post_code = self.post_code
        country: Union[Unset, str] = UNSET
        if not isinstance(self.country, Unset):
            country = self.country.value

        foreign_country = self.foreign_country

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if line1 is not UNSET:
            field_dict["line1"] = line1
        if line2 is not UNSET:
            field_dict["line2"] = line2
        if line3 is not UNSET:
            field_dict["line3"] = line3
        if line4 is not UNSET:
            field_dict["line4"] = line4
        if line5 is not UNSET:
            field_dict["line5"] = line5
        if post_code is not UNSET:
            field_dict["postCode"] = post_code
        if country is not UNSET:
            field_dict["country"] = country
        if foreign_country is not UNSET:
            field_dict["foreignCountry"] = foreign_country

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        line1 = d.pop("line1", UNSET)

        line2 = d.pop("line2", UNSET)

        line3 = d.pop("line3", UNSET)

        line4 = d.pop("line4", UNSET)

        line5 = d.pop("line5", UNSET)

        post_code = d.pop("postCode", UNSET)

        _country = d.pop("country", UNSET)
        country: Union[Unset, Country]
        if isinstance(_country, Unset):
            country = UNSET
        else:
            country = Country(_country)

        foreign_country = d.pop("foreignCountry", UNSET)

        address = cls(
            line1=line1,
            line2=line2,
            line3=line3,
            line4=line4,
            line5=line5,
            post_code=post_code,
            country=country,
            foreign_country=foreign_country,
        )

        return address
